<?php
//
// COUNTER CLASS, IMPLEMENTATION SHOULD BE AS SINGLETON
// kh_mod 0.1.0 rc1, add, 0.4.2 changed
class MG2Counter {

	var $mg2;					// @var object reference
	var $pathCounter;			// @var string, counter file path (flat file only)
	var $pathChronicle;		// @var string, chronicle file path (flat file only)
	var $type;					// @var string, database type (flat file or sql)
	var $counterData;			// @var array, counter data array
	var $chronicleData;		// @var array, chronicle data array
	var $isCounterData;		// @var bool, read status counter data (true/false)
	var $isChronicleData;	// @var bool, read status chronicle data (true/false)
	var $changedRCs;			// @var array, changed data records

	// INITIALIZE COUNTER
	function MG2Counter(&$parent, $restoreDB) {
		$this->mg2				  = &$parent;
		$this->type				  = ($this->mg2->sqldatabase)? 'sql':'flat';
		$this->pathCounter	  = DATA_FOLDER .'mg2db_counter.php';
		$this->pathChronicle   = DATA_FOLDER .'mg2db_chronicle.php';
		$this->isCounterData	  = $restoreDB ? false:$this->loadDatabase();
		$this->isChronicleData = false;
		$this->changedRCs      = array();
	}

	// GET NUMBER OF COUNTER RECORDS
	function getNumCounters() {
		return ($this->isCounterData === true)?
				 count($this->counterData)
				 :
				 false;
	}

	// GET NUMBER OF CHRONICLE RECORDS
	function getNumChronicles() {

		if ($this->isChronicleData === true) {
			$numRecords = 0;
			foreach (array('daily', 'year', 'lastOpt') as $dateType) {
				if (empty($this->chronicleData[$dateType])) continue;

				// COUNT DATA RECORDS
				foreach ($this->chronicleData[$dateType] as $date=>$chronicleRC) {
					$numRecords += isset($chronicleRC['itemClicks']) ?
										count($chronicleRC['itemClicks'])
										:
										1;
				}
			}
		}
		else {
			$numRecords = false;
		}

		return $numRecords;
	}

	// GET NUMBER OF CLICKS OF ONE ITEM
	function getNumClicks($itemID) {
		return (isset($this->counterData[$itemID][1]))?
				 $this->counterData[$itemID][1]
				 :
				 '0';
	}

	// GET NUMBER OF COMMENTS
	function getNumComments($itemID) {
		return (isset($this->counterData[$itemID][5]) && (int)$this->counterData[$itemID][5] === 1)?
				 (int)$this->counterData[$itemID][4]
				 :
				 -1;	// number of comments unknown
	}

	// SET CLICK AND COMMENT COUNTER
	function setDatabase($itemID, $clicks=-1, $comments=-1, $write=true) {
		if (!$this->isCounterData) 							return;
		settype($itemID, 'integer');	if ($itemID < 1)	return;

		// ARE THERE CLICK OR COMMENT COUNTS TO WRITE
		$counterToWrite = 0;

		// EXISTS COUNTER ENTRY NOT YET
		if (!isset($this->counterData[$itemID])) {
			$this->counterData[$itemID] = array (
														$itemID,			// item id
														0,					// num mouse clicks
														0,					// date first click
														0					// date last click
													);
		}
		else $this->counterData[$itemID][0] = $itemID;

		// SET CLICK COUNTER, BUT COUNTER LOCKED (ADMIN MODE)
		if ($clicks === 1 && empty($_SESSION[GALLERY_ID]['adminmode'])) {

			// INIT TIMESTAMP
			$timestamp = time();

			// INIT FIRST ENTRY
			if (empty($this->counterData[$itemID][1])) {
				$this->counterData[$itemID][1] = 0;				// counter
				$this->counterData[$itemID][2] = $timestamp;	// first entry
			}

			// INCREASE COUNTER DATA BY ONE
			$this->counterData[$itemID][1]++;					// counter
			$this->counterData[$itemID][3] = $timestamp;		// latest entry
			$counterToWrite      			 = 1;					// write click data
		}

		// SET COMMENT COUNTER
		if ($comments >= 0) {
			$this->counterData[$itemID][4] = $comments;		// comments
			$this->counterData[$itemID][5] = 1;					// verified counter ok

			// SAVE COMMENT NUMBER DIRECTLY
			if ($write === true || $counterToWrite) {
				$counterToWrite |= 2;								// write comment data
			}
			// SAVE COMMENT NUMBER LATER
			else {
				$this->changedRCs[] = $itemID;
			}
		}

		if ($counterToWrite) {

			// SAVE CLICK AND COMMENT DATA
			$writeOK = $this->writeDatabase('upd', array($itemID));

			// INCREASE CHRONICLE COUNTER, IF CLICK COUNTER SET
			if ($writeOK > 0 && ($counterToWrite & 1)) {
				$this->setChronicleData($itemID);
			}
		}

		return ($counterToWrite && $writeOK > 0)? true:false;
	}

	// UPDATE DATABASE CHANGES
	function updateRecords() {

		if (!empty($this->changedRCs)) {
			if ($this->writeDatabase('upd', $this->changedRCs) > 0) {
				$this->changedRCs = array();
			}
		}
	}

	// INCREASE CHRONICAL COUNTER BY ONE
	function setChronicleData($itemID) {

		$date = (int)date('Ymd');
		return ($this->type === 'sql')?
				 $this->setChronicleDataSQL($itemID, $date)
				 :
				 $this->setChronicleDataFlatfile($itemID, $date);
	}

	// INCREASE CHRONICAL COUNTER BY ONE - SQL
	function setChronicleDataSQL($itemID, $date) {

		$query = 'INSERT INTO `mg2db_chronicle` (`date`, `itemID`, `itemClicks`, `lastClick`, `dateCode`)
					 VALUES ("'.$date.'", '.$itemID.', 1, '. time() .', 1)
					 ON DUPLICATE KEY UPDATE
						`itemClicks` = `itemClicks` + 1,
						`lastClick`  = '. time() .'
					 ';
		$incOK = mg2mysql_query($query);

		return ($incOK && mg2mysql_affected_rows())? true:false;
	}

	// INCREASE CHRONICAL COUNTER BY ONE - FLATFILE
	function setChronicleDataFlatfile($itemID, $date) {

		// INIT WRITE STATUS
		$writeOK = -1;

		// LOAD CHRONICLE DATA
		if ($this->loadChronicleData()) {

			// CHRONICLE DATA TODAY NOT YET OPTIMIZED?
			if (!isset($this->chronicleData['lastOpt'][$date]) 		&&
				 is_array($this->chronicleData['daily'])					&&
				 ($days = array_keys($this->chronicleData['daily'])))
			{
				$this->mergeChronicleDataFlat(array(min($days)), $date);
			}

			// INCREASE CHRONICLE BY ONE
			if (empty($this->chronicleData['daily'][$date]['itemClicks'][$itemID])) {
				$this->chronicleData['daily'][$date]['itemClicks'][$itemID] = 1;
			}
			else {
				$this->chronicleData['daily'][$date]['itemClicks'][$itemID]++;
			}

			// SET TIMESTAMP
			$this->chronicleData['daily'][$date]['lastClick'][$itemID] = time();

			// UPDATE CHRONICLE DATA
			$writeOK = $this->writeChronicle();
		}

		return ($writeOK > 0)? true:false;
	}

	// DELETE RECORDS WITHOUT RELATIONS TO 'all_items' (ADMIN ONLY)
	function delResidueCounterData() {

		if (!$this->isCounterData) return;

		$deleteOK  = -1;
		$deleteIDs = array_diff(
								array_keys($this->counterData),
								array_keys($this->mg2->all_items)
						 );

		if ($nToDelete = count($deleteIDs)) {
			foreach ($deleteIDs as $itemID) {
				unset($this->counterData[$itemID]);
			}
			$deleteOK = $this->writeDatabase('del', $deleteIDs);
		}

		return ($deleteOK !== -1) ? $nToDelete:false;
	}

	// DELETE RECORDS WITHOUT RELATIONS TO 'all_items' (ADMIN ONLY)
	function delResidueChronicleData() {

		return ($this->type === 'sql')?
				 $this->delResidueChronicleDataSQL()
				 :
				 $this->delResidueChronicleDataFlat();
	}

	// DELETE RECORDS WITHOUT RELATIONS TO 'mg2db_items' - SQL
	function delResidueChronicleDataSQL() {

		$query = 'DELETE FROM t1
					 USING `mg2db_chronicle` t1 LEFT JOIN `mg2db_items` t2
					 ON     t1.itemID = t2.itemID
					 WHERE  `dateCode` <> 4
					 AND    t2.itemID IS NULL';
		$delOK = mg2mysql_query($query);

		// SET CHRONICLE DATA DIRTY
		if (($numDelIDs = mg2mysql_affected_rows()) > 0) $this->isChronicleData = false;

		return $delOK ? $numDelIDs:false;
	}

	// DELETE RECORDS WITHOUT RELATIONS TO 'all_items' - Flatfile
	function delResidueChronicleDataFlat() {

		if (!$this->loadChronicleData()) return 0;

		$itemIDs  = array_keys($this->mg2->all_items);
		$nDeleted = 0;
		foreach(array('daily', 'year') as $dataType) {
			foreach(array_keys($this->chronicleData[$dataType]) as $date) {
				$deleteIDs = array_diff(
										array_keys($this->chronicleData[$dataType][$date]['itemClicks']),
										$itemIDs
								 );
				if ($nToDelete = count($deleteIDs)) {
					if ($nToDelete === count($this->chronicleData[$dataType][$date]['itemClicks'])) {
						unset($this->chronicleData[$dataType][$date]);
					}
					else {
						foreach ($deleteIDs as $itemID) {
							unset($this->chronicleData[$dataType][$date]['itemClicks'][$itemID]);
							unset($this->chronicleData[$dataType][$date]['lastClick'][$itemID]);
						}
					}
					$nDeleted += $nToDelete;
				}
			}
		}
		$delOK = $nDeleted ? $this->writeChronicle():0;

		return ($delOK !== -1) ? $nDeleted:false;
	}

	// MERGES OLDER CHRONICLE DATA AND DELETES THOSE FROM LAST YEAR (ADMIN ONLY)
	function optimizeChronicleData() {

		return ($this->type === 'sql')?
				 $this->optimizeChronicleDataSQL()
				 :
				 $this->optimizeChronicleDataFlat();
	}

	// MERGE DATA WHICH ARE OLDER THAN ONE MONTH INTO 'YEAR' RECORDS
	function optimizeChronicleDataSQL() {

		// TODAY ALREADY OPTIMIZED?
		$query = 'SELECT IF(`date` = CURDATE(),1,0) `current`
					 FROM  `mg2db_chronicle`
					 WHERE `dateCode` = 4 LIMIT 1';
		if (($result      = mg2mysql_query($query))        &&
			 ($lastOptDate = mg2mysql_fetch_assoc($result)) &&
			  $lastOptDate['current'])
		{
			return; // already optimzed
		}

		// ALLOW INVALID DATE ENTRIES
		mg2mysql_query('SET @@session.sql_mode = ALLOW_INVALID_DATES');

		// DELETE EXPIRED DATA RECORDS
		$monthBefore = $this->getRecentDate('monthBefore');	// integer
		$thisYear	 = $this->getRecentDate('thisYear');		// integer
		$query		 = 'DELETE FROM `mg2db_chronicle`
							 WHERE (`dateCode` = 1 AND `date` < CAST("'. min($thisYear, $monthBefore) .'" AS DATETIME))
							 OR	 (`dateCode` = 2 AND `date` < CAST("'. $thisYear .'" AS DATETIME))
							 OR    (`dateCode` = 4)';
		mg2mysql_query($query);

		// MERGE DATA RECORDS
		if (($records = $this->getSQLChronicleMergeValues($monthBefore)) !== false) {

			$query = 'INSERT INTO `mg2db_chronicle` (`date`, `itemID`, `itemClicks`, `lastClick`, `dateCode`)
						 VALUES '. $records .'
						 ON DUPLICATE KEY UPDATE
							`itemClicks` = `itemClicks` + VALUES(`itemClicks`),
							`lastClick`  = VALUES(`lastClick`),
							`dateCode`   = VALUES(`dateCode`)';
			if (mg2mysql_query($query)) {

				// DELETE MERGED RECORDS
				$query = 'DELETE FROM `mg2db_chronicle`
							 WHERE `dateCode` = 1 AND `date` < CAST("'. $monthBefore .'" AS DATETIME)';
				mg2mysql_query($query);
			}
		}

		// SET CHRONICLE DATA DIRTY
		$this->isChronicleData = false;
	}

	// GET ALL CHRONICLE RECORDS BEFORE TIMESTAMP '$monthBefore'
	function getSQLChronicleMergeValues($monthBefore) {

		// INIT RETURN VALUE
		$return = false;

		// GET ALL CHRONICLE DATA WHICH ARE OLDER THEN ONE MONTH
		$query = 'SELECT DATE_FORMAT(`date`, "%Y-00-00") `year`, `date`, `itemID`, `itemClicks`, `lastClick`
					 FROM  `mg2db_chronicle`
					 WHERE `dateCode` = 1 AND `date` < "'. $monthBefore .'"';

		if ($result = mg2mysql_query($query)) {

			// SET LAST OPTIMIZED DATE
			$toMerge = array('(CURDATE(), 0, 0, NULL, 4)');

			// MERGE DATA RECORDS
			while ($record = mg2mysql_fetch_assoc($result)) {
				$toMerge[] = sprintf('("%s", %d, %d, %s, 2)',
									 $record['year'],
									 $record['itemID'],
									 $record['itemClicks'],
									 isset($record['lastClick']) ? (int)$record['lastClick']:'NULL'
								 );
			}

			// BUILD RETURN STRING
			$return = implode(',', $toMerge);
		}

		return $return;
	}

	// MERGE DATA WHICH ARE OLDER THAN ONE MONTH INTO 'YEAR' RECORDS
	function optimizeChronicleDataFlat($doWrite=true) {

		if (!$this->loadChronicleData()) return;

		// TODAY ALREADY OPTIMIZED?
		$date = (int)date('Ymd');
		if (isset($this->chronicleData['lastOpt'][$date])) {
			return; // already optimzed
		}

		// SET NEW OPTIMIZED DATE
		$this->chronicleData['lastOpt'] = array($date=>$date);

		// INIT DELETE COUNTER
		$nDeleted = 0;

		// DOES ANY DAILY DATE EXIST
		if (isset($this->chronicleData['daily']) && is_array($this->chronicleData['daily'])) {
			$dailyChronData = array_keys($this->chronicleData['daily']);
			$nDeleted      += $this->mergeChronicleDataFlat($dailyChronData, $date);
		}

		// DELETE 'year' CHRONICLE RECORDS WHICH ARE OLDER THAN THE CURRENT YEAR
		if (isset($this->chronicleData['year']) && (count($this->chronicleData['year']) > 1)) {

			$monthBefore  = $this->getRecentDate('monthBefore', $date);
			$thisYear     = $this->getRecentDate('thisYear', $date);
			$deleteBefore = min($thisYear, $monthBefore);

			foreach (array_keys($this->chronicleData['year']) as $year) {
				if ($year < $deleteBefore) {
					$nDeleted += count($this->chronicleData['year'][$year]['itemClicks']);
					unset($this->chronicleData['year'][$year]);
				}
			}
		}

		$delOK = $doWrite ? $this->writeChronicleFlatfile():0;
		return ($delOK !== -1) ? $nDeleted:false;
	}

	// MERGE 'daily' CHRONICLE RECORDS to 'year' DATA WHICH ARE OLDER THAN ONE MONTH
	function mergeChronicleDataFlat($dailyChronData, $date) {

		// GET ONE MONTH BEFORE AND THIS YEAR
		$monthBefore = $this->getRecentDate('monthBefore', $date);
		$thisYear    = $this->getRecentDate('thisYear', $date);

		$nDeleted = 0;
		foreach ($dailyChronData as $chronDate) {
			if ($chronDate > $monthBefore) continue;

			// GET THE YEAR OF THE CURRENT TO MERGE DATE
			$chronYear = (int)(substr($chronDate, 0, 4) .'0000');

			// MERGE DAILY DATA RECORD INTO THIS YEAR
			if ($chronYear === $thisYear) {

				// '$chronYear' NOT YET SET
				if (!isset($this->chronicleData['year'][$chronYear])) {
					$this->chronicleData['year'][$chronYear] = array(
																				'itemClicks'=>array(),
																				'lastClick' =>array()
																			 );
				}

				$itemIDs = array_keys($this->chronicleData['daily'][$chronDate]['itemClicks']);
				foreach ($itemIDs as $itemID) {

					// 'itemClicks' NOT YET SET
					if (!isset($this->chronicleData['year'][$chronYear]['itemClicks'][$itemID])) {
						$this->chronicleData['year'][$chronYear]['itemClicks'][$itemID] = 0;
					}

					$itemClicks = (int)$this->chronicleData['daily'][$chronDate]['itemClicks'][$itemID];
					$lastClick  = isset($this->chronicleData['daily'][$chronDate]['lastClick'][$itemID]) ?
									  (int)$this->chronicleData['daily'][$chronDate]['lastClick'][$itemID]
									  :
									  null;
					$this->chronicleData['year'][$chronYear]['itemClicks'][$itemID] += $itemClicks;
					$this->chronicleData['year'][$chronYear]['lastClick'][$itemID]   = $lastClick;
				}
			}

			// DELETE 'daily' CHRONICLE RECORDS WHICH ARE OLDER THAN ONE MONTH
			$nDeleted += count($this->chronicleData['daily'][$chronDate]['itemClicks']);
			unset($this->chronicleData['daily'][$chronDate]);
		}

		return $nDeleted;
	}

	// READ COUNTER DATA
	function loadDatabase() {
		$this->counterData = array();		// counter data array
		$this->changedRCs  = array();		// not yet saved record IDs
		return ($this->type === 'sql')?
				 $this->loadDatabaseSQL()
				 :
				 $this->loadDatabaseFlatfile();
	}

	// READ COUNTER DATA - SQL
	function loadDatabaseSQL() {
		$query = 'SELECT `itemID`,
							  `itemClicks`,
							  `firstClick`,
							  `lastClick`,
							  `commentsNum`,
							  `commentsOk`
					 FROM `mg2db_counter` WHERE `itemID` > 0';
		if (!$result = mg2mysql_query($query)) return false;

		while ($record = mg2mysql_fetch_row($result)) {
			$this->counterData[(int)$record[0]] = $record;
		}
		return true;
	}

	// READ COUNTER DATA - FLAT FILE
	function loadDatabaseFlatfile($pathDB='default') {

		// COUNTER FILE PATH
		$pathDB = ($pathDB === 'default')? $this->pathCounter : $pathDB;

		if (is_file($pathDB)) {

			// LOCK COUNTER FILE
			$fp     = fopen($pathDB, 'rb');
			$locked = flock($fp, LOCK_SH);

			/* LOAD COUNTER DATA
			 * $loadedData might be:
			 *    false        - read error
			 *    empty string - blocked by a writing operation
			 *    loaded data  - OK
			 */
			$loadedData = fread($fp, filesize($pathDB));

			// UNLOCK COUNTER FILE
			if ($locked) flock($fp, LOCK_UN);
			fclose($fp);

			// IF LOADING FAILS
			if (empty($loadedData))	return false;

			foreach(explode("\n", $loadedData) as $line) {
				if ($line === '' || $line{0} !== '#')	continue;	// no data row
				$line   = trim(substr($line, 1));
				$record = explode("\t", $line);
				if (($itemID = (int)$record[0]) < 1)	continue; 	// invalid item id
				$this->counterData[$itemID] = $record;
			}
		}
		return true;
	}

	// READ CHRONICLE DATA
	function loadChronicleData() {

		// CHECK IF DATA ALREADY LOADED
		if ($this->isChronicleData) return true;

		// INIT CHRONICLE DATA ARRAY
		$this->chronicleData = array (
										  'daily' => array(),				// clicks counted on a daily basis
										  'year'	 => array(),				// clicks counted on this year
									  );

		// LOAD DATA
		$this->isChronicleData = ($this->type === 'sql')?
										 $this->loadChronicleSQL()
										 :
										 $this->loadChronicleFlatfile();

		return $this->isChronicleData;
	}

	// READ CHRONICLE DATA - SQL (ADMIN ONLY)
	function loadChronicleSQL() {

		// SELECT CHRONICLE DATA
		$query = 'SELECT DATE_FORMAT(`date`, "%Y%m%d") `day`,
							  `itemID`,
							  `itemClicks`,
							  `lastClick`,
							  `dateCode`
					 FROM `mg2db_chronicle` WHERE 1';
		if (!$result = mg2mysql_query($query)) return false;

		// CONVERT CHRONICLE DATA INTO THE ARRAY 'chronicleData'
		$dateMap = array(1=>'daily', 2=>'year', 4=>'lastOpt');
		while ($record = mg2mysql_fetch_assoc($result)) {
			$date     = (int)$record['day'];
			$itemID   = (int)$record['itemID'];
			$dataCode = (int)$record['dateCode'];
			if (!isset($dateMap[$dataCode])) continue;

			$dataType = $dateMap[$dataCode];
			if ($dataType === 'lastOpt') {
				$this->chronicleData[$dataType] = array($date=>$date);
			}
			else {
				$this->chronicleData[$dataType][$date]['itemClicks'][$itemID] = (int)$record['itemClicks'];
				if (isset($record['lastClick'])) {
					$this->chronicleData[$dataType][$date]['lastClick'][$itemID] = (int)$record['lastClick'];
				}
			}
		}

		return true;
	}

	// READ CHRONICLE DATA - FLAT FILE
	function loadChronicleFlatfile($filePath='default') {

		// GET FILE PATH
		$filePath = ($filePath === 'default')?
						$this->pathChronicle
						:
						$filePath;

		// EXISTS FILE THAN READ DATA
		if (is_file($filePath)) {

			// LOCK COUNTER CHRONICLE FILE
			$fp     = fopen($filePath, 'rb');
			$locked = flock($fp, LOCK_SH);

			/* LOAD CHRONICLE DATA
			 * $loadedData might be:
			 *    false            - read error
			 *    empty string     - blocked by a writing operation
			 *    serialized array - OK
			 */
			$loadedData = fread($fp, filesize($filePath));

			// UNLOCK COUNTER CHRONICLE FILE
			if ($locked) flock($fp, LOCK_UN);
			fclose($fp);

			// IF LOADING FAILS
			if (empty($loadedData)) return false;

			// UNSERIALISE LOADED DATA AND APPLY THEM
			$unserializedData = unserialize($loadedData);
			if (is_array($unserializedData))	{
				$this->chronicleData = &$unserializedData;
			}
		}

		return true;
	}

	// WRITE COUNTER DATA
	function writeDatabase($action='all', $itemIDs=false) {
		if ($this->isCounterData !== true) return -1;

		return ($this->type === 'sql')?
				 $this->writeDatabaseSQL($action, $itemIDs)
				 :
				 $this->writeDatabaseFlatfile();
	}

	// WRITE COUNTER DATA - SQL
	function writeDatabaseSQL($action, $itemIDs) {
		// USE DIFFERENT RECORDS AND $records IS NOT AN ARRAY?
		if ($action !== 'all' && !is_array($itemIDs)) return -1;

		$table  = 'mg2db_counter';
		$result = -1;
		switch ($action) {
			case 'new':
			case 'upd':
				$values = array();
				foreach ($itemIDs as $id) {
					if (!array_key_exists($id, $this->counterData)) continue;	// no record?
					$record	 = $this->counterData[$id];
					$values[] = $this->getSQLCounterValues($record);
				}
				$query = 'REPLACE INTO `'. $table .'` VALUES'. implode(',', $values);
				if (mg2mysql_query($query)) $result = mg2mysql_affected_rows();
				break;
			case 'all': if(!defined('USER_ADMIN')) break;
				$query   = 'TRUNCATE TABLE `'. $table .'`'; if (!mg2mysql_query($query)) break;
				$towrite = 0;
				$written = 0;
				foreach ($this->counterData as $record) {
					$towrite++;
					$query = 'INSERT INTO `'. $table .'` VALUES'. $this->getSQLCounterValues($record);
					if (mg2mysql_query($query)) $written++;
				}
				if ($written === $towrite) $result = $written;
				break;
			case 'del': if(!defined('USER_ADMIN')) break;
				$query = 'DELETE FROM `'. $table .'` WHERE itemID IN ('. implode(',', $itemIDs) .')';
				if (mg2mysql_query($query)) $result = mg2mysql_affected_rows();
				break;
			default: $this->mg2->status = 'Error in counter database';
		}
		return $result;
	}

	// GET VALUES TO WRITE INTO SQL DATABASE
	function getSQlCounterValues(&$record) {
		return sprintf('(%d ,%d ,%d ,%d ,%d ,%d ,%f ,%d)',
					 $record[0],									// item id
					 $record[1],									// num mouse clicks
					 $record[2],									// first click
					 $record[3],									// last click
					 isset($record[4]) ? $record[4]:0,		// num comments
					 isset($record[5]) ? $record[5]:0,		// comment valid (boolean)
					 isset($record[6]) ? $record[6]:0,		// avg rating
					 isset($record[7]) ? $record[7]:0		// num rating
				 );
	}

	// WRITE COUNTER DATA - FLATFILE
	function writeDatabaseFlatfile() {
		$buffer  = "-- MG2/kh_mod click and comment counter --\n";
		$records = 0;
		$writeOK = false;
		foreach($this->counterData as $line) {
			if (!is_array($line)) continue;	// no record?
			$buffer.= '#';
			$buffer.= implode("\t", $line);
			$buffer.= "\n";
			$records++;
		}

		// NO COUNTER RECORDS
		if ($records === 0)
		{
			$writeOK = $this->delDataFile($this->pathCounter);
		}
		// WRITE COUNTER FILE
		elseif (($fp = @fopen($this->pathCounter, 'wb'))	||
				  (rename($this->pathCounter, sprintf('%s%s_mg2db_counter.php',
															BACKUP_FOLDER,
															time()
														))					&&
					($fp = fopen($this->pathCounter, 'wb')))
				 )
		{
			flock($fp, LOCK_EX);					// do an exclusive lock
			ftruncate($fp, 0);					// truncate to zero length
			$writeOK = fwrite($fp, $buffer);
			flock($fp, LOCK_UN);					// release the lock
			fclose($fp);
		}
		return $writeOK ? $records:-1;
	}

	// WRITE CHRONICLE COUNTER DATA
	function writeChronicle() {
		if ($this->isChronicleData !== true) return -1;

		return ($this->type === 'sql')?
				 $this->writeChronicleSQL()
				 :
				 $this->writeChronicleFlatfile();
	}

	// WRITE CHRONICLE COUNTER DATA - SQL (ADMIN ONLY)
	function writeChronicleSQL() {
		if (!defined('USER_ADMIN')) return -1;

		// INIT START VALUES
		$result = -1;
		$table  = 'mg2db_chronicle';

		// DELETE OLD ENTRIES
		$delQuery = 'TRUNCATE TABLE `'. $table .'`';
		if (mg2mysql_query($delQuery)) {

			// ALLOW INVALID DATE ENTRIES
			mg2mysql_query('SET @@session.sql_mode = ALLOW_INVALID_DATES');

			// PREPARE WRITING
			$writeOK = true;
			$written = 0;
			foreach (array('daily', 'year', 'lastOpt') as $dateType) {
				if (empty($this->chronicleData[$dateType])) continue;

				// WRITE DATA RECORDS
				foreach (array_keys($this->chronicleData[$dateType]) as $date) {
					if (!$values = $this->getSQLChronicleValues($date, $dateType)) continue;

					$insQuery = 'INSERT INTO `'. $table .'` VALUES'. $values;
					if (!mg2mysql_query($insQuery)) {
						$this->mg2->log('ERROR: In "writeChronicleSQL" ('. mg2mysql_error() .')');
						mg2mysql_query($delQuery);
						$writeOK = false;
						break 2;						// completely write break
					}

					$written += mg2mysql_affected_rows();
				}
			}
		}

		return $writeOK ? $written:-1;
	}

	// GET VALUES TO WRITE INTO SQL 'mg2db_chronicle' TABLE
	function getSQLChronicleValues($date, $dateType) {

		// INIT RETURN VALUE
		$records  = array();

		if (preg_match('/^(\d{4})((\d{2})(\d{2}))?$/', $date, $treffer)) {
			$year		 = $treffer[1];
			$month	 = isset($treffer[3]) ? $treffer[3]:'00';
			$day		 = isset($treffer[4]) ? $treffer[4]:'00';
			$dateCode = array('daily'=>1, 'year'=>2, 'lastOpt'=>4);

			if ($dateType === 'lastOpt') {
				$dateRC = array('itemClicks'=>array(0));	// dummy array
			}
			else {
				$dateRC = &$this->chronicleData[$dateType][$date];
			}

			foreach (array_keys($dateRC['itemClicks']) as $itemID) {

				$itemClicks = $dateRC['itemClicks'][$itemID];
				$lastClick	= isset($dateRC['lastClick'][$itemID]) ?
								  (int)$dateRC['lastClick'][$itemID]
								  :
								  'NULL';

				// BUILD DATA RECORD
				$records[]	= sprintf('("%04d-%02d-%02d", %d, %d, %s, %d)',
										$year,
										$month,
										$day,
										$itemID,
										$itemClicks,			// 0 ... n
										$lastClick,				// null | timestamp
										$dateCode[$dateType]	// 1 | 2 | 4
								  );
			}
		}

		return implode(',', $records);
	}

	// WRITE CHRONICLE COUNTER DATA - FLATFILE
	function writeChronicleFlatfile() {

		// INIT START VALUES
		$writeOK = false;
		$records = $this->getNumChronicles();	// false, 0...n

		// NO COUNTER RECORDS
		if ($records === 0)
		{
			$writeOK = $this->delDataFile($this->pathChronicle);
		}
		// SAVE CHRONICLE DATA
		elseif (($fp = @fopen($this->pathChronicle, 'wb'))	||
			 (rename($this->pathChronicle, sprintf('%s%d_mg2db_chronicle.php',
														 BACKUP_FOLDER,
														 time()
													 ))					&&
			  ($fp = fopen($this->pathChronicle, 'wb')))
			)
		{
			flock($fp, LOCK_EX);					// do an exclusive lock
			ftruncate($fp, 0);					// truncate to zero length
			$writeOK = fwrite($fp, serialize($this->chronicleData));
			flock($fp, LOCK_UN);					// release the lock
			fclose($fp);
		}

		return $writeOK ? $records:-1;
	}

	// DO DELETE ONE OF COUNTER FILE
	function delDataFile($filepath) {

		if  (!defined('USER_ADMIN')) return false;

		// THERE'S NOTHING TO WRITE, DO DELETE CHRONICLE FILE
		if (is_file($filepath))
		{
			$logMsg = ($deleteOK = unlink($filepath))?
						 'File \''. basename($filepath) .'\' was deleted!'
						 :
						 'File \''. basename($filepath) .'\' could not be deleted!';
			$this->mg2->log($logMsg);
		}
		// THERE'S NEIHTER TO WRITE NOR A CHRONICLE FILE
		else
		{
			$deleteOK = true;
		}

		return $deleteOK;
	}

	// GET PAST DATE OF PERIOD AS INTEGER
	function getRecentDate($timePeriod='tenDays', $date=false) {

		// GET CURRENT DATE
		if ($date == false) $date = date('Ymd');
		$m = substr($date, 4, 2);
		$d = substr($date, 6, 2);
		$Y = substr($date, 0, 4);

		// BUILD DATE (BEGIN OF PERIOD)
		if ($timePeriod === 'tenDays') {
			$recentDate = date('Ymd', mktime(0, 0, 0, (int)$m, $d-10, (int)$Y));
		}
		elseif ($timePeriod === 'lastDays') {
			$recentDate = date('Ymd', mktime(0, 0, 0, (int)$m, $d-28, (int)$Y));
		}
		elseif ($timePeriod === 'monthBefore') {
			$t = date('t', mktime(0, 0, 0, (int)$m, 0, (int)$Y));    // days last month
			$recentDate = date('Ymd', mktime(0, 0, 0, (int)$m-1, min($d,$t), (int)$Y));
		}
		elseif ($timePeriod === 'thisMonth') {
			$recentDate = $Y . $m . '01';
		}
		elseif ($timePeriod === 'thisYear') {
			$recentDate = $Y .'0000';
		}
		else {
			$recentDate = 0;
		}

		return (int)$recentDate;
	}
}
?>