<?php
//
// ADMIN COUNTER CLASS, IMPLEMENTATION SHOULD BE AS SINGLETON
// kh_mod 0.4.0 rc1 add
class MG2AdminCounter extends MG2Counter {

	// INITIALIZE ADMIN COUNTER
	function MG2AdminCounter(&$parent, $restoreDB) {
		$this->MG2Counter($parent, $restoreDB);
	}

	// DELETE IMAGE/VIDEO/SOUND ENTRIES
	function deleteRecords($deletedIDs) {
		if (!is_array($deletedIDs)) return false;

		$this->delResidueCounterData();
		$this->delResidueChronicleData();
		$this->optimizeChronicleData();
	}

	// RESET COMMENT COUNTER
	function resetComments($action='upd') {
		$changes = array();
		foreach($this->counterData as $itemID=>$row) {
			// NO ENTRY OR ALREADY SET INVALID TO 0
			if (empty($row[5]))	continue;

			$this->counterData[$itemID][5] = 0;		// set comment counter invalid
			$changes[] = $itemID;
		}
		return ($action==='upd' && empty($changes))?
				 true
				 :
				 $this->writeDatabase($action, $changes);
	}

	// SWITCH DATABASE
	function switchDatabase($db) {
		if (($db = strtolower($db)) === $this->type) return -1;

		// INIT RETURN VALUES
		$counterOK = $chronicleOK = -1;

		// VALID DATABASE SWITCH
		if ($db === 'sql' || $db === 'flat') {

			// SWITCH COUNTER AND CHRONICLE DATA
			$this->optimizeChronicleData();			// merge chronicle data which are older than one month
			$this->loadChronicleData();				// load chronicle data if they aren't up to date
			$this->type  = $db;
			$counterOK   = $this->writeDatabase('all');
			$chronicleOK = $this->writeChronicle();
		}

		return ($counterOK > -1 && $chronicleOK > -1)?
				  array($counterOK, $chronicleOK)
				  :
				  -1;
	}

	// BACKUP DATABASE
	function backupDatabase($db) {

		// INIT START VALUES
		$writeOK  = -1;
		$tempPath = DATA_FOLDER . 'mg2db_'. $db .'_temp.php';

		switch ($db) {
			case 'counter':
				if ($this->isCounterData !== true)	break;
				$this->pathCounter = $tempPath;
				$writeOK = $this->writeDatabaseFlatfile();
				break;
			case 'chronicle':
				$this->optimizeChronicleData();
				if ($this->loadChronicleData() !== true) break;
				$this->pathChronicle = $tempPath;
				$writeOK = $this->writeChronicleFlatfile();
		}

		return ($writeOK > -1) ? $writeOK : false;
	}

	// RESTORE DATABASE
	function restoreDatabase($restoreFilePath) {

		// INIT COUNTER/CHRONICLE DATA
		$this->counterData	  = array();
		$this->chronicleData   = array();
		$this->isCounterData	  = false;
		$this->isChronicleData = false;
		$numCounter				  = -2;
		$numChronicle			  = -2;
		$resCounter            =  0;
		$resChronicle			  =  0;

		do {
			// IMPORT COUNTER DATA
			$restoreCounterPath = sprintf($restoreFilePath, 'counter');
			if (!$this->loadDatabaseFlatfile($restoreCounterPath))	break;	// backup file read error

			$this->isCounterData = true;
			$numCounter				= 0;														// 0 no data to restore
			if (count($this->counterData) < 1)								break;	// no data records
			if (($numCounter = $this->resetComments('all')) < 1)		break;	// -1 write error, 1 to n number of records

			$resCounter = $this->delResidueCounterData();							// delete orphaned records in "mg2db_counter"

			// IMPORT CHRONICLE DATA
			$restoreChroniclePath  = sprintf($restoreFilePath, 'chronicle');
			$this->loadChronicleFlatfile($restoreChroniclePath);

			$this->isChronicleData = true;
			$this->updateChronicleStruct();												// update chronical structure
			$this->optimizeChronicleDataFlat(false);									// merge data which are older than one month
			if (($numChronicle = $this->writeChronicle()) < 1)			break;	// -1 write error, 0 to n number of records

			$resChronicle = $this->delResidueChronicleData();						// delete orphaned records in "mg2db_chronicle"
		}
		while(0);

		return array (
					'counter'	=> array('total' => $numCounter,   'residue' => $resCounter),
					'chronicle' => array('total' => $numChronicle, 'residue' => $resChronicle)
				 );
	}

	/**
	 * DO UPDATE CHRONICAL STRUCTURE
	 * ACTUALLY FOR kh_mod 0.4.0 rc1 TO 0.4.1
	 */
	function updateChronicleStruct() {

		foreach (array('daily', 'year') as $level) {
			if (!isset($this->chronicleData[$level]))	continue;
			if (!is_array($this->chronicleData[$level])) {
				$this->chronicleData[$level] = array();
				continue;
			}

			foreach (array_keys($this->chronicleData[$level]) as $day) {
				if (isset($this->chronicleData[$level][$day]['itemClicks'])) continue;

				$chronicleRC = is_array($this->chronicleData[$level][$day]) ?
									$this->chronicleData[$level][$day]
									:
									array();

				$this->chronicleData[$level][$day]               = array();			// delete possible old values
				$this->chronicleData[$level][$day]['lastClick']  = array();			// timestamp last click array
				$this->chronicleData[$level][$day]['itemClicks'] = $chronicleRC;	// set new structure for clicks
			}
		}
	}

	// IMPORT A COMPLETE COUNTER DATABASE (CONVERT ONLY)
	function importDatabase($database=false) {
		if (!is_array($database)) return false;

		$this->counterData = $database;
		return (($records = $this->writeDatabase()) === -1)?
				 false
				 :
				 $records;
	}
}