<?php
//
// UPLOAD AND IMPORT CLASS, IMPLEMENTATION SHOULD BE AS SINGLETON
// kh_mod 0.4.1, add
class MG2Import {

	var $mg2;				// parent object reference
	var $modify;			// upload/import options
	var $ItemProcess;		// instance of 'MG2ItemProcess'
	var $cssRotateClass; // according to the Exif orientation

	// INITIALIZE IMPORT
	function MG2Import(&$parent) {
		$this->mg2 = &$parent;
		$this->mg2->getMemoryStatus();
		$this->setImportOptions();
		$this->setItemAutoID();
	}

	// BUILD INSTANCE OF CLASS 'MG2ItemProcess'
	function instantiateImgProcess() {

		require_once(INC_FOLDER .'mg2admin_itemprocess.php');
		if (!isset($this->ItemProcess)) {
			$this->ItemProcess = new MG2ItemProcess($this->mg2);
		}
	}

	// GET UPLOAD/IMPORT OPTIONS
	function setImportOptions() {

		// SET UPLOAD AND IMPORT OPTIONS
		if (isset($_POST['action']) || isset($_POST['display'])) {
			$this->modify  = (isset($_POST['thumb'])  && (int)$_POST['thumb']	 === 1)? 1:0;
			$this->modify |= (isset($_POST['medium']) && (int)$_POST['medium'] === 1)? 2:0;
			$this->modify |= (isset($_POST['delete']) && (int)$_POST['delete'] === 1)? 4:0;
			$this->modify |= (isset($_POST['rotate']) && (int)$_POST['rotate'] === 1)? 8:0;
		}
		elseif (isset($_SESSION[GALLERY_ID]['import_modify'])) {
			$this->modify  = (int)$_SESSION[GALLERY_ID]['import_modify'];
		}
		else {
			$this->modify  = 3;	// default
		}
		$_SESSION[GALLERY_ID]['import_modify'] = $this->modify;

		// SET UPLOAD AND SERVER DIRECTORY
		if (isset($_POST['importFrom'])) {
			$_SESSION[GALLERY_ID]['import_subdir'] = trim($_POST['importFrom']);
		}
		elseif (!isset($_SESSION[GALLERY_ID]['import_subdir'])) {
			$_SESSION[GALLERY_ID]['import_subdir'] = '';
		}
	}

	// SET THE LATEST USED ITEM ID
	function setItemAutoID() {
		$this->mg2->itemAutoID = $this->mg2->all_items ?
									max(array_keys($this->mg2->all_items))
									:
									0;
	}

	// GET THE CURRENT SUBDIR TO IMPORT
	function getCurrentSubdir() {
		return $_SESSION[GALLERY_ID]['import_subdir'];
	}

	// UPLOAD ITEMS - ACTION
	function uploadItems($folderID) {

		// CHECK FOLDER ID
		if (!isset($this->mg2->all_folders[$folderID])) {
			$this->mg2->displayStatus($this->mg2->lang['nofolderid'] .' #'. $folderID, 3);
			return false;	// corrupted folder id
		}

		// GET SUB DIRECTORY FOR IMPORT (SERVER)
		$subdir = trim($_POST['targetdir']);
		if (!$this->mg2->getImportDir($subdir)) {
			$this->mg2->displayStatus($this->mg2->lang['invalidimportpath'], 2);
			$this->mg2->log('No upload possible, invalid server path!');
			return false;	// no import
		}

		// START UPLOAD AND IMPORT
		$this->mg2->log('HTML-Upload and Import had been started:');
		$this->instantiateImgProcess();							// instantiate class 'MG2ItemProcess'

		// INIT START VALUES
		$importedFiles = $this->getImportedItems($subdir);	// get already imported
		$allfiles		= 0;											// number of uploaded files without error
		$overridden 	= 0;											// number of overidden files
		$fileExists		= 0;											// number of already existing files
		$renamed			= 0;											// number of renamed files
		$toImport		= array();									// IDs of files to be imported

		// RESET ITEM TASK LIST (SESSION)
		$this->ItemProcess->delItemProcessList();

		// GET UPLOADED FILES
		foreach (array_keys($_FILES['file']['name']) as $uplID) {
			if (trim($_FILES['file']['name'][$uplID]) === '') 			continue;	// empty upload entry

			// COUNT ALL UPLOADED FILES
			$allfiles++;

			// GET VALUES OF UPLOAD FORM
			if (!$uploadedFile = $this->getUploadFormValues($uplID))	continue;	// forbidden input

			$overwrite = $uploadedFile['overwrite'];
			$toFolder  = $uploadedFile['toFolder'];
			$tmpPath   = $uploadedFile['tmpPath'];
			$tmpSize   = $uploadedFile['tmpSize'];
			$fileName  = $uploadedFile['fileName'];
			$fileExt   = $uploadedFile['extension'];
			$filePath  = $this->mg2->getFilePathByName($fileName, $subdir);

			// IMPORT THUMB OR MEDIUM IMAGES ONLY IF OVERWRITE
			$regexp = '/_(thumb|medium)(\.'. $fileExt .')$/i';
			preg_match($regexp, $fileName, $isThumbMedium);
			if ($isThumbMedium) {
				if (!$overwrite)													continue;

				// MAIN FILE THAT BELONGS TO THE THUMB
				$fileName = preg_replace($regexp, '$2', $fileName, 1);
			}

			// START ITEM - LOG MESSAGE
			$this->mg2->log(sprintf('for file \'%s\' (%d Bytes)',
									$fileName,
									$tmpSize
							));

			// START ITEM FILE IMPORT, IF NEW FILE?
			if (!is_file($filePath) && !$isThumbMedium) {
				if (!$this->moveUploadedFile($tmpPath, $filePath))		continue;	// can't import new file
				if (in_array($fileName, $importedFiles, true))			continue;	// item is already imported
				$uploaded++;

				$itemID = $this->buildItemRecord($toFolder, $fileName, $subdir, $tmpSize);
			}
			// OVERWRITE EXISTING FILE
			elseif ($overwrite) {
				$itemID = array_search($fileName, $importedFiles, true);
				if (!$itemID)														continue;	// item isn't imported
				if (!$this->moveUploadedFile($tmpPath, $filePath))		continue;	// can't import new file
				$this->mg2->log(' * existing file has been overridden');
				$overridden++;

				if ($isThumbMedium[1] === 'medium')							continue;	// nothing else to do

				// SET NEW THUMB SIZE IF CHANGED
				if ($isThumbMedium[1] === 'thumb') {
					list($newW, $newH) = getimagesize($filePath);
					$oldW = (int)$this->mg2->all_items[$itemID][10];
					$oldH = (int)$this->mg2->all_items[$itemID][11];
					if ($oldW === $newW && $oldH === $newH)				continue;	// nothing else to do

					$this->mg2->all_items[$itemID][10] = $newW;
					$this->mg2->all_items[$itemID][11] = $newH;
				}
				// CREATE NEW THUMB AND MEDIUM FILE FOR ITEM
				else {
					$this->mg2->deleteThumbMedium($fileName, $subdir);
					$this->ItemProcess->addItemProcessTask($itemID, $this->modify);
				}
			}
			// FILE NAME ALREADY EXISTS, BUT DIFFERENT FILE (SIZE)?
			elseif (filesize($filePath) !== $tmpSize) {
				$regexp = '/(_?\((\d{1,12})\))?\.'. $fileExt .'$/i';
				if (!preg_match($regexp, $fileName, $appendix))			continue;	// file syntax error

				$i = ((int)$appendix[2] > 1000000)? 0:(int)$appendix[2];
				$baseName = substr($fileName, 0, -strlen($appendix[0]));
				$changeTo = $baseName .'_(%d).'. $fileExt;
				do {
					$fileName = sprintf($changeTo, ++$i);
					$filePath = $this->mg2->getFilePathByName($fileName, $subdir);
				}
				while(is_file($filePath));
				if (!$filePath)													continue;	// file path error
				if (!$this->moveUploadedFile($tmpPath, $filePath))		continue;	// can't import new file
				$this->mg2->log(' * file has been renamed to \''. $fileName .'\'');
				$renamed++;

				$itemID = $this->buildItemRecord($toFolder, $fileName, $subdir, $tmpSize);
			}
			// UPLOADED FILE EXISTS BUT NOT ENTERED IN DATABASE?
			else {
				$fileExists++;
				if ($itemID = array_search($fileName, $importedFiles, true)) {
					$folderName = $this->mg2->getFolderName((int)$this->mg2->all_items[$itemID][1]);
					$this->mg2->log(' * file already has been imported into folder \''. $folderName .'\'');
					continue;
				}

				$this->mg2->log(' * file already exists in directory \''. dirname($filePath) .'\'');
				$itemID = $this->buildItemRecord($toFolder, $fileName, $subdir, $tmpSize);
			}

			$toImport[$itemID]      = $itemID;		// to avoid duplicate id
			$importedFiles[$itemID] = $fileName;	// to avoid same file names			
		}

		// COUNT TO IMPORT ITEMS
		$numToImport = count($toImport);

		// WRITE ITEMS DATA AND UPDATE
		$importOK = $numToImport && ($this->mg2->write_iDB('new', $toImport) > 0);

		// SET LAST IMPORTED ITEMS LIST
		if ($importOK) $_SESSION[GALLERY_ID]['importedItems'] = $toImport;

		// MESSAGE FOR RENAMED FILES
		$renameMsg = $renamed ?
						 sprintf(', %s</div><div>%d %s',
								$this->mg2->lang['therefrom'],
								$renamed,
								$this->mg2->lang['filesrenamed']			
						 )
						 :
						 '';

		// DISPLAY UPLOAD MESSAGE
		if ($numUploaded = $uploaded + $renamed + $overridden) {
			$this->mg2->status = sprintf(
											'<div>%d&nbsp;%s%s</div>',
											$numUploaded,
											$this->mg2->lang['filesuploaded'],
											$importOK ? '':$renamedMsg
										);
		}
		if ($fileExists > 0) {
			$this->mg2->status.= sprintf('<div style="color:teal;">%s %d %s</div>',
											$this->mg2->lang['notice'],
											$fileExists,
											$this->mg2->lang['alreadyexists']
										);
		}
		if ($numToImport) {
			$this->mg2->status.= '<div style="margin: 0 auto 1em auto;border-bottom:1px solid #9EBE9E;width:30%">&nbsp;</div>';
		}

		// DISPLAY IMPORT MESSAGE
		if ($importOK) {
			$this->mg2->status.= sprintf(
											'<div>%d&nbsp;%s%s</div>',
											$numToImport,
											$this->mg2->lang['filesimported'],
											$renamedMsg
										);
		}
		elseif ($fileExists < 1) {
			$message = $this->mg2->lang['nofilestoimport'];
			if ($allfiles > 0) {
				$langChunk = $allfiles === 1 ? 'one_forbidden_file':'n_forbidden_files';
				$message  .= sprintf('<div>('. $this->mg2->lang[$langChunk] .')</div>',
									 $allfiles
								 );
			}
			$this->mg2->displayStatus($message, 1);
		}

		if ($overridden > 0) {
			$this->mg2->status.= sprintf('<div style="color:teal">%s %d %s</div>',
											$this->mg2->lang['notice'],
											$overridden,
											'duplicate file(s) have been overridden'
										);
		}
		if ($numToImport > 0) {
			$iDB_error = (!$importOK)?
							 sprintf('<div style="color:red">%s %s</div>', $this->mg2->lang['error'], $this->mg2->lang['iDB_error'])
							 :
							 '';
			$this->mg2->status.= '<div style="margin: 0 auto 1em auto;border-bottom:1px solid #9EBE9E;width:30%">&nbsp;</div>';
			$this->mg2->status.= sprintf('%s<div style="margin:8px 0"><a href="%s?fID=%d">%s \'%s\'</a></div>',
											$iDB_error,
											ADMIN_INDEX,
											$folderID,
											$this->mg2->lang['backtofolder'],
											$this->mg2->getFolderName($folderID, true)
										);
		}
		$this->mg2->displayStatus(); flush();

		// THERE ARE NEW ITEMS IMPORTED
		if ($importOK) {
			$numAllItems = count($this->mg2->all_items);
			$this->mg2->log(sprintf('Summary: Imported %d file(s). Now, the gallery contains %d items',
									$numToImport,
									$numAllItems
								));
			$this->ItemProcess->doItemProcessList(false);
		}
		// THERE ARE IMPORT TASKS BUT NOT SAVED
		elseif ($numToImport) {
			$this->ItemProcess->delItemProcessList();
		}
		// NOTHING TO IMPORT
		else {
			$this->mg2->log('No files to import!');
		}

		return $importOK;
	}

	// COPY UPLOADED ITEM TO IMAGE DIRECTORY
	function moveUploadedFile($tmpPath, $filePath) {

		$uploadOK = move_uploaded_file($tmpPath, $filePath);
		if ($uploadOK) {
			$this->mg2->log(' * file uploaded to directory \''. dirname($filePath) .'\'');
		}

		return $uploadOK;
	}

	// READ AND VERIFY UPLOAD FORM VALUES
	function getUploadFormValues($item) {

		// GET FORM VALUES
		$errNumber = (int)$_FILES['file']['error'][$item];				// upload error number
		$tmpPath   = trim($_FILES['file']['tmp_name'][$item]);		// temp file path (absolute)
		$tmpSize   = (int)$_FILES['file']['size'][$item];				// uploaded file size
		$fileName  = trim($_FILES['file']['name'][$item]);				// uploaded file name
		$toFolder  = !empty($_POST['uploadto'][$item]) ?				// target folder id
						 (int)$_POST['uploadto'][$item]
						 :
						 (int)$_GET['fID'];	// fall back
		$overwrite = isset($_POST['overwrite'. $item]) ? true:false;

		// THE UPLOADED FILE EXCEEDS THE 'upload_max_filesize' DIRECTIVE
		if ($errNumber === UPLOAD_ERR_INI_SIZE) {
			$max_upload = ini_get('upload_max_filesize');
			$this->mg2->displayStatus(sprintf($this->mg2->lang['uploadfile_oversized'],
													$fileName,
													preg_replace('/(\d+)M$/','${1} MByte', $max_upload),
													'upload_max_filesize'
											 ), 2);
			return false;
		}

		// THE FILE WAS ONLY PARTIALLY UPLOADED
		if ($errNumber === UPLOAD_ERR_PARTIAL) {
			$this->mg2->displayStatus(sprintf($this->mg2->lang['partially_uploaded'],
													$fileName
											 ), 2);
			return false;
		}

		// ANY OTHER UPLOAD ERROR HAS HAPPEND
		if ($errNumber !== UPLOAD_ERR_OK) {
			$this->mg2->displayStatus(sprintf($this->mg2->lang['upload_error'],
													$fileName,
													$errNumber
											 ), 2);
			return false;
		}

		// CHECK FORBIDDEN CHARACTERS
		if (preg_match('/[:;<>\/\\\|\?"]/', $fileName)) {
			$this->mg2->displayStatus(sprintf('\'%s\' %s',
													$fileName,
													$this->mg2->lang['renamefailure']
											 ), 2);
			return false;
		}

		// CHECK FILE EXTENSION
		$extension = substr(strrchr($fileName, '.'), 1);
		if (!$this->mg2->checkFileExtension($extension)) {
			$this->mg2->displayStatus(sprintf($this->mg2->lang['void_extension'],
													$fileName
											 ), 2);
			return false;
		}

		// FOLDER DOESN'T EXIST
		if (!isset($this->mg2->all_folders[$toFolder])) {
			$this->mg2->displayStatus(sprintf($this->mg2->lang['notargetfolder'],
													$fileName
											 ), 2);
			return false;
		}

		return array(
					'toFolder'  => $toFolder,
					'tmpPath'   => $tmpPath,
					'tmpSize'   => $tmpSize,
					'fileName'  => $fileName,
					'extension' => $extension,
					'overwrite' => $overwrite
				 );
	}

	// IMPORT START - ACTION
	function importStart($importTofID) {

		// CHECK FOLDER ID
		if (!isset($this->mg2->all_folders[$importTofID])) {
			$this->mg2->displayStatus($this->mg2->lang['nofolderid'] .' #'. $importTofID, 3);
			return false;	// invalid folder id
		}

		// GET IMPORT PAHT FOR ITEMS (SERVER)
		$subdir = $this->getCurrentSubdir();
		if (!$importDir = $this->mg2->getImportDir($subdir)) {
			$this->mg2->displayStatus($this->mg2->lang['invalidimportpath'], 2);
			$this->mg2->log('No import possible, invalid server path!');
			return 0;		// no import
		}

		// IMPORT SERVER DIRECTORY STRUCTURE?
		if (!empty($_POST['dirStruct'])) $this->importDirStructure($importTofID, $subdir);

		// START IMPORT
		$this->mg2->log('Import had been started:');	// log start entry
		$this->instantiateImgProcess();					// instantiate class 'MG2ItemProcess'

		// GET TO IMPORT FILES
		$toImport = $this->getImportFormValues($importTofID, $importDir, $subdir);

		// WRITE ITEMS DATA AND UPDATE
		$numImported = ($toImport && ($this->mg2->write_iDB('new', $toImport) > 0))?
							count($toImport)
							:
							0;

		// SET LAST IMPORTED ITEMS LIST
		if ($numImported) $_SESSION[GALLERY_ID]['importedItems'] = $toImport;

		// DISPLAY STATUS MESSAGE
		$folderName  = $this->mg2->getFolderName($importTofID, true);
		$numAllItems = count($this->mg2->all_items);
		$logMessage  = sprintf('Summary: Imported %d file(s) to \'%s\'. Now, the Gallery contains %d items',
								$numImported,
								$folderName,
								$numAllItems
							);
		$this->mg2->log($logMessage);
		if ($numImported < 1) {
			$dsplMessage = $this->mg2->lang['nofilestoimport'];
			$errorType	 = 1;
		} else {
			$dsplMessage = sprintf('%d %s<div style="margin-top:8px"><a href="%s?fID=%d">%s \'%s\'</a></div>',
									$numImported,
									$this->mg2->lang['filesimported'],
									ADMIN_INDEX,
									$importTofID,
									$this->mg2->lang['backtofolder'],
									$folderName
								);
			$errorType 	 = 0;
		}
		$this->mg2->displayStatus($dsplMessage, $errorType);

		// START IMAGE PROCESSING
		if ($numImported) $this->ItemProcess->doItemProcessList(false);

		return $numImported;
	}

	// READ AND VERIFY IMPORT FORM VALUES
	function getImportFormValues($importTofID, $importDir, $subdir) {

		// INIT START VALUES
		$importedFiles = $this->getImportedItems($subdir);			// get already imported
		$allowedExt    = $this->getAllowedExtensions();				// allowed file extensions
		$toImport      = array();
		$formData      = array();

		// RESET ITEM TASK LIST (SESSION)
		$this->ItemProcess->delItemProcessList();

		// GET IMPORT ITEMS
		if (isset($_POST['fileNames']) && is_array($_POST['fileNames'])) {
			$formData = $_POST['fileNames'];
		}

		foreach ($formData as $base64Name) {

			// CHECK FILE PATH
			$fileName = base64_decode($base64Name);
			$filePath = $importDir .'/'. $fileName;
			if (!is_file($filePath)) {
				$this->mg2->log('but couldn\'t find file \''. $filePath .'\'');
				continue;
			}

			// GET FILE SIZE
			$fileSize = filesize($filePath);

			// START ITEM - LOG MESSAGE
			$this->mg2->log(sprintf('for file \'%s\' (%d Bytes)',
									$fileName,
									$fileSize
							));

			// CHECK FILE EXTENSION
			$extension = strtolower(substr(strrchr($fileName, '.'), 1));
			if (!isset($allowedExt[$extension]))				continue;

			// ITEM ALREADY IMPORTED
			if (in_array($fileName, $importedFiles, true))	continue;

			// BUILD NEW ITEM RECORD
			$itemID = $this->buildItemRecord($importTofID, $fileName, $subdir, $fileSize);
			$toImport[$itemID] = $itemID;
		}

		return $toImport;
	}

	// GET CURRENTLY ALLOWED FILE EXTENSIONS
	function getAllowedExtensions() {

		$allowedExt = array();
		foreach ($this->mg2->getSupportedExtensions() as $idx=>$name) {
			if ($this->mg2->extensions & $idx) {
				$allowedExt[$name] = $idx;
			}
		}

		return $allowedExt;
	}

	// BUILD A NEW ITEM RECORD - ACTION
	function buildItemRecord($folderID, $fileName, $subdir, $fileSize) {

		// GET FILE PATH
		$filePath = $this->mg2->getFilePathByName($fileName, $subdir);
		@chmod($filePath, 0644);

		// BUILD A NEW ITEM RECORD
		$itemID = ++$this->mg2->itemAutoID;
		$this->mg2->all_items[$itemID] = array(
														$itemID,					// item autoid				[0]
														$folderID,				// contained in folder	[1]
														'',						// item headline			[2]
														'',						// item description		[3]
														time(),					// publishing date		[4]
														1,							// sorting position		[5]
														$fileName,				// file name				[6]
														$subdir,					// subdir path				[7]
														-1,						// item width				[8]
														-1,						// item height				[9]
														-1,						// thumb width				[10]
														-1,						// thumb height			[11]
														$fileSize,				// item file size			[12]
														'',						// exif date				[13]
														'',						// Photographer			[14]
														0,							// Bookmarked				[15]
														$this->modify << 12	// file code				[16]
												 );

		// SET TASK TO CREATE THUMBNAIL RSP. MEDIUM IMAGE
		$this->ItemProcess->addItemProcessTask($itemID, $this->modify);

		// LOG MESSAGES
		$currDate	= date('Ymd, H:i:s');
		$logMessage.= $currDate .' -  * built data record (ID #'. $itemID .')' ."\n";
		$logMessage.= $currDate .' -  * added data record to database'. "\n";
		$logMessage.= $currDate .' -  * added to process task list'. "\n";	
		$this->mg2->log($logMessage, true);

		return $itemID;
	}

	// IMPORT THE SERVER DIRECTORY STRUCTURE
	function importDirStructure($infolder=1, $subdir='') {

		$sublevel  = preg_match_all('#/[^/]+#', $subdir, $matches);
		$Treeview  = &$this->mg2->getInstance('MG2Treeview');
		$subdirs	  = $Treeview->getItemsSubdirs($subdir, $sublevel);
		$fID		  = $infolder;
		$records   = array();
		$structure = array();
		foreach($subdirs as $dir) {

			// DIRECTORY SUB LEVEL
			$newlevel = $dir['level'];
			if ($newlevel > $sublevel) $structure[$newlevel] = $fID;
			$infolder = $structure[$newlevel];
			$sublevel = $newlevel;

			// FOLDER ALREADY EXISTS
			$sameNames = array();
			foreach ($this->mg2->all_folders as $key=>$record) {
				if ((int)$record[1] !== $infolder)					continue;
				if (strcasecmp($record[2], $dir['name']) !== 0)	continue;

				$sameNames[] = $key;
			}
			if (count($sameNames) === 1) {
				$fID = $sameNames[0];
				continue;
			}

			// CREATE FOLDER RECORD
			$fID = ++$this->mg2->folderAutoID;
			$this->mg2->all_folders[$fID] = array(
								$fID,				// folderID						[0]
								$infolder,		// in folder (#id)			[1]
								$dir['name'],	// folder name					[2]
								'',				// introtext					[3]
								time(),			// timestamp (publish off)	[4]
								1,					// folder position			[5]
							  -1,					// imageID for folder icon	[6]
								6,					// folder sort mode			[7]
								'',				// password						[8]
								0,					// number of cols				[9]
								0,					// number of rows				[10]
								0,					// folder type					[11]
								''					// folder url					[12]
							);
			$newFolderIDs[] = $fID;
		}

		// SAVE NEW FOLDERS AND DISPLAY MESSAGE, kh_mod 0.4.1 changed 
		if ($numNewFolders = count($newFolderIDs)) {
			// IMPORT NEW FOLDERS OK
			if ($this->mg2->write_fDB('new', $newFolderIDs) > 0) {
				$message = sprintf($this->mg2->lang['foldersimported'], $numNewFolders, count($subdirs));
				$errtype = 0;
			}
			// IMPORT ERROR
			else {
				$message = sprintf($this->mg2->lang['folderimport_error'], $numNewFolders);
				$errtype = 3;
			}
		}
		else {
			$message = sprintf($this->mg2->lang['nofolderstoimport']);
			$errtype = 1;
		}
		$this->mg2->displayStatus($message, $errtype);
	}

	// GET ALL NOT YET IMPORTED FILES OF DIRECTORY $subdir
	function getNotImportedFiles($subdir) {

		$newFiles  = array();
		$importDir = $this->mg2->getImportDir($subdir);
		if ($handle = opendir($importDir)) {
			$allowedExt = $this->getAllowedExtensions();
			while (false !== ($fileName = readdir($handle))) {
				$extension = substr(strrchr($fileName, '.'), 1);
				$extension = strtolower($extension);	
				if (empty($extension))											continue;	// extension empty
				if (!isset($allowedExt[$extension]))						continue;	// extension forbidden
				if (strpos($fileName, '_medium.') !== false)				continue;	// medium image
				if (strpos($fileName, '_thumb.')  !== false)				continue;	// thumb image

				$newFiles[] = $fileName;
			}
			closedir($handle);
		}

		// REMOVE ALREADY IMPORTED ITEMS
		return array_diff(
					 $newFiles,
				 	 $this->getImportedItems($subdir)
				 );
	}

	// GET ALL ALREADY IMPORTED FILES OF DIRECTORY $subdir
	function getImportedItems($subdir) {

		$allItems = array();
		foreach ($this->mg2->all_items as $itemID=>$record) {
			if ($record[7] !== $subdir) continue;

			$allItems[$itemID] = $record[6];
		}
		return $allItems;
	}

	// GET SRC VALUE FOR THUMB TAG
	function getThumbSrcValue($fName) {

		// INIT CSS ROTATE CLASS
		$this->cssRotateClass = '';

		// MSIE 7 OR LESS
		return isset($_SESSION[GALLERY_ID]['ie'][7]) ?
				 $this->getThumbURI($fName)
				 :
				 $this->getThumbData($fName);
	}

	// GET THUMB BASE64 DATA
	function getThumbData($fName) {

		//INIT START VALUES
		$thumbData = false;
		$itemCode  = $this->mg2->getContentCode(substr(strrchr($fName, '.'), 1));

		do {
			// JPEG IMAGE
			if ($itemCode & 2) {
				$thumbType = 'jpeg';
				$thumbData = $this->getExifThumb($fName);
				break;
			}

			// VIDEO FILE
			$metaCode = $itemCode & 448;
			if ($metaCode === $this->mg2->contentCodeTable('video')) {
				$thumbType = 'gif';
				$thumbData = file_get_contents(ADMIN_IMAGES .'video.gif');
				break;
			}

			// AUDIO FILE
			if ($metaCode === $this->mg2->contentCodeTable('audio')) {
				$thumbType = 'gif';
				$thumbData = file_get_contents(ADMIN_IMAGES .'audio.gif');
				break;
			}
		}
		while(0);

		return $thumbData ?
				 sprintf('data:image/%s;base64,%s', $thumbType, base64_encode($thumbData))
				 :
				 false;
	}

	// GET THUMB URI
	function getThumbURI($fName) {

		//INIT RETURN VALUE
		$thumbPath = false;
		$itemCode  = $this->mg2->getContentCode(substr(strrchr($fName, '.'), 1));

		do {
			// JPEG IMAGE WITH EXIF THUMB
			if ($itemCode & 2 && $this->getExifThumb($fName)) {
				$thumbPath = sprintf('%sadmin_thumbstream.php?f=%s&s=%s&v=%s',
											ADMIN_FOLDER,
											rawurlencode($fName),
											rawurlencode($this->getCurrentSubdir()),
											$this->mg2->setVerificationToken($fName)
								 );
				break;
			}

			// VIDEO FILE
			$metaCode = $itemCode & 448;
			if ($metaCode === $this->mg2->contentCodeTable('video')) {
				$thumbPath = ADMIN_IMAGES .'video.gif';
				break;
			}

			// AUDIO FILE
			if ($metaCode === $this->mg2->contentCodeTable('audio')) {
				$thumbPath = ADMIN_IMAGES .'audio.gif';
				break;
			}
		}
		while(0);

		return $thumbPath;
	}

	// GET THE EXIF THUMB OF A PICTURE
	function getExifThumb($fName) {

		include_once(INC_FOLDER .'mg2_exif.php');
		$subdir    = $this->getCurrentSubdir();
		$imagePath = $this->mg2->getFilePathByName($fName, $subdir);
		$Exif		  = &$this->mg2->getInstance('MG2Exif');
		$exifData  = $Exif->getExifData($imagePath);
		$this->setRotateClassName($exifData['Orientation']);

		return !empty($exifData['ThumbData']) ?
				 $exifData['ThumbData']
				 :
				 false;
	}

	// SET ROTATE CSS CLASS DUE TO EXIF ORIENTATION
	function setRotateClassName($exifOrient) {

		if (!isset($exifOrient)) return;

		if ($exifOrient === 'Right top') {
			$this->cssRotateClass = 'imgRotate90';
		}
		elseif ($exifOrient === 'Left bottom') {
			$this->cssRotateClass = 'imgRotate270';
		}
		elseif ($exifOrient === 'Bottom right') {
			$this->cssRotateClass = 'imgRotate180';
		}
	}
}
?>