<?php
//
// ITEM PROCESSING CLASS, IMPLEMENTATION SHOULD BE AS SINGLETON
// kh_mod 0.4.1, add
class MG2ItemProcess {

	var $mg2;	// parent object reference

	// INITIALIZE IMPORT
	function MG2ItemProcess(&$parent) {
		$this->mg2 = &$parent;
	}

	// ADD ITEM RECORD TO RESIZE TASKLIST
	function addItemProcessTask($itemID, $modify) {

		// SET MODIFY IN DATA RECORD
		(int)$this->mg2->all_items[$itemID][16] |= $modify << 12;

		$rebuildTasks = &$_SESSION[GALLERY_ID]['rebuildTasks'];
		$rebuildTasks[$itemID] = array(
											'modify' => $modify,
											'fName'  => trim($this->mg2->all_items[$itemID][6]),
											'subdir' => trim($this->mg2->all_items[$itemID][7])
										 );
	}

	// (RE)BUILD THUMB AND MEDIUM IMAGES ACTION
	function doItemProcessList($doSuccessMsg=true) {

		// INIT START VALUES
		$this->mg2->getMemoryStatus();
		$tasksDone  = 0;
		$tasksTotal = 0;

		// START IMAGE PROCESSING
		$this->mg2->log('Image processing had been started:');
		foreach ($this->mg2->getItemProcessList() as $itemID=>$task) {
			if (!isset($this->mg2->all_items[$itemID])) {
				$this->deleteSingleTask($itemID);
				$this->mg2->displayStatus(sprintf($this->mg2->lang['nopictureid'], $itemID), 3);
				continue;
			}

			// COUNT TASKS
			$tasksTotal++;

			// GET ITEM DATA
			$fileName = $task['fName'];
			$subdir	 = $task['subdir'];
			$modify   = $task['modify'];

			// GET FILEPATH
			$filePath = $this->mg2->getFilePathByName($fileName, $subdir);
			if (!is_readable($filePath)) {
				$this->mg2->log(' * ERROR: File \''. $filePath .'\' not readable!');
				$this->mg2->displayStatus(sprintf($this->mg2->lang['imagenotread'], $filePath), 3);
				continue;
			}

			// ITEM PROCESSING LOG ENTRY
			$this->mg2->log(sprintf('for item ID #%d (\'%s\')',
								$itemID,
								$filePath
							));

			// GET ITEM SIZE AND CREATE THUMBNAIL RSP. MEDIUM IMAGE
			$finfo = $this->resizeImage($itemID, $fileName, $filePath, $modify);

			// IF THERE ARE NO FATAL ERRORS
			if (!isset($finfo['error'][3])) {

				// UPDATE DATABASE
				if ($finfo['rotate'] || ($finfo['scaled'] & 4)) clearstatcache();
				$imageRC     = &$this->mg2->all_items[$itemID];
				$imageRC[6]  = $fileName;
				$imageRC[7]  = $subdir;
				$imageRC[8]  = $finfo['item']['width'];
				$imageRC[9]  = $finfo['item']['height'];
				$imageRC[10] = $finfo['thumb']['width'];
				$imageRC[11] = $finfo['thumb']['height'];
				$imageRC[12] = filesize($filePath);
				$imageRC[13] = empty($imageRC[13]) ? '':(int)$imageRC[13];
				$imageRC[14] = empty($imageRC[14]) ? '':trim($imageRC[14]);
				$imageRC[15] = empty($imageRC[15]) ?  0:(int)$imageRC[15];
				$imageRC[16] = $finfo['code'];

				// DO UPDATE ITEM DATABASE
				if ($this->mg2->write_iDB('upd', array($itemID)) < 1) {
					$this->mg2->displayStatus($this->mg2->lang['iDB_error'], 3);
					$this->mg2->log(' * ERROR: Couldn\'t write item data!');
					continue;
				}
			}

			// DELETE COMPLETED TASK
			$this->deleteSingleTask($itemID);

			// FATAL ERROR
			if (isset($finfo['error'][3])) {
				$this->mg2->displayStatus($this->mg2->htmlchars2entities($finfo['error'][3]), 3);
				continue;
			}

			// ITEM TYPE/MEMORY/SCALE/ROTATE WARNING
			if (isset($finfo['error'][2])) {
				$this->mg2->displayStatus($this->mg2->htmlchars2entities($finfo['error'][2]), 2);
			}
			// DO COUNT SUCCESSFUL TASKS
			else {
				$tasksDone++;
			}

			// DISPLAY REBUILT MESSAGE ONLY
			if ($doSuccessMsg) {
				$bullet  = '<span style="display:block;margin-left:1.5em;text-align:left">'. $this->mg2->lang['bullet'] .' %s (%d x %d)</span>';
				$thumb	= sprintf($bullet, $this->mg2->lang['thumbrebuilt'], $finfo['thumb']['width'], $finfo['thumb']['height']);
				$medium	= sprintf($bullet, $this->mg2->lang['mediumrebuilt'], $finfo['medium']['width'], $finfo['medium']['height']);
				$image	= sprintf($bullet, $this->mg2->lang['imagescaled'], $finfo['item']['width'], $finfo['item']['height']);
				$message = sprintf(
									'<span style="display:inline-block;">
										<span style="display:block;">%s \'%s\'</span>
										%s
										%s
										%s
									</span>
									',
									$this->mg2->lang['rebuildsuccess'],
									$filePath,
									($finfo['scaled'] & 1) ? $thumb :'',
									($finfo['scaled'] & 2) ? $medium:'',
									($finfo['scaled'] & 4) ? $image :''
							);
				$this->mg2->displayStatus($message);
			}
		}

		// CONCLUSION LOG ENTRY
		$this->mg2->log('Summary: Processed '. $tasksDone .' of '. $tasksTotal .' item(s) successfully.');

		return $tasksDone;
	}

	// DELETE ONE IMPORT/REBUILD TASK FROM LIST
	function deleteSingleTask($itemID) {
		unset($_SESSION[GALLERY_ID]['rebuildTasks'][$itemID]);
	}

	//	DELETE ALL IMPORT/REBUILD TASKS FROM LIST
	function delItemProcessList() {
		$_SESSION[GALLERY_ID]['rebuildTasks'] = array();
	}

	// DELETE IMPORT/REBUILD TASKS FROM ITEM RECORDS
	function delAllProcessTasks() {

		// DELETE TASKS IN DATABASE
		$itemIDs = array_keys($_SESSION[GALLERY_ID]['rebuildTasks']); 
		foreach($itemIDs as $itemID) {
			(int)$this->mg2->all_items[$itemID][16] &= 8191;		// remain the first 13 bits
		}

		// WRITE ITEM DATABASE
		if ($this->mg2->write_iDB('upd', $itemIDs) < 1) {
			$this->mg2->displayStatus($this->mg2->lang['iDB_error'], 3);
		}
		else {
			$this->delItemProcessList();
		}
	}

	// GET ITEM SIZE AND CREATE THUMBNAIL RSP. MEDIUM IMAGE
	function resizeImage($itemID, $fileName, $imgFilePath, $modify=3) {

		// SET TIME LIMIT
		@set_time_limit(90);

		// INIT RETURN VALUES
		$result = array (
						'item'     => array('width'=>-1, 'height'=>-1),
						'thumb'    => array('width'=>-1, 'height'=>-1),
						'code'     => 0,
						'scaled'   => 0,
						'rotate'   => 0,
						'error'    => array()
					 );

		// UNSUPPORTED ITEM TYPE
		if (!$itemMetrics = $this->getItemMetrics($imgFilePath)) {
			$result['error'][3] = sprintf($this->mg2->lang['unsupported_filetype'], $fileName);
			return $result;
		}

		// SET ITEM METRICS
		$result['code'] = $itemMetrics['itemCode'];
		$result['item'] = array(
									'width'  => $itemMetrics['itemInfo'][0],	// original width
									'height' => $itemMetrics['itemInfo'][1]	// original height
								);

		// IF NOT IMAGE PROCESSING (VIDEO, SOUND)
		if ($itemMetrics['process'] === false)	return $result;

		// SET PROCESS CONSTRAINTS
		$modify &= $itemMetrics['process'];

		// GET THE IMAGE ORIGINAL SIZE (ALIAS)
		$imgWidth  = $itemMetrics['itemInfo'][0];
		$imgHeight = $itemMetrics['itemInfo'][1];

		// SET ROTATION INIT VALUES
		$rotateDeg = 0;
		$errLevel  = $modify & 64 ? 3:2;	// if rotation !important

		// IF JPEG IMAGE THEN COMPUTE EXIF-DATE
		$exifOK = false;
		if ($itemMetrics['formatFct'] === 'imagejpeg')
		{
			$exifOK = $this->mg2->computeExifDate(array($itemID=>$imgFilePath));
			if (($modify & 8)	&&
				  $exifOK		&&
				  isset($this->mg2->MG2Exif->exifData['Orientation']))
			{
				$modify &= 15;	// delete explicit rotate order
				$exifOrient = $this->mg2->MG2Exif->exifData['Orientation'];
				if     ($exifOrient === 'Right top')    $rotateDeg = -90;
				elseif ($exifOrient === 'Bottom right') $rotateDeg = 180;
				elseif ($exifOrient === 'Left bottom')  $rotateDeg =  90;
			}
		}

		// SET EXPLICIT ROTATE ORDER
		switch ($modify >> 4 & 3) {
			case 1: $rotateDeg = -90; break;
			case 2: $rotateDeg = 180; break;
			case 3: $rotateDeg =  90; break;
		}

		// BUILD ROTATE IMAGE RECORD
		$create = array();
		if ($rotateDeg) {

			// DOES NOT EXIT FUNCTION 'imagerotate'
			if (!function_exists('imagerotate')) {
				$this->mg2->log(' * WARNING: GD function \'imagerotate\' was being missed!');
				$result['error'][$errLevel] = $this->mg2->lang['missed_imagerotate'];
				$rotateDeg = 0;

				// IF ROTATION !IMPORTANT
				if ($errLevel === 3) return $result;
			}
			else {
				$create['item'] = array(8, $imgWidth, $imgHeight, false);
			}
		}

		// BUILD MEDIUM IMAGE RECORD
		$mediumMaxSize = (int)$this->mg2->mediumimage;
		if (($modify & 2) && $mediumMaxSize > 0)
		{
			// OVERWRITE (DELETE) THE ORIGINAL IMAGE?
			if ($modify & 4) {
				$create['item']   = array($rotateDeg ? 10:2, $mediumMaxSize, $mediumMaxSize, false);
			}
			else {
				$create['medium'] = array(2, $mediumMaxSize, $mediumMaxSize, false);
			}
		}

		// BUILD THUMB IMAGE RECORD
		$create['thumb'] = array(
									$modify & 1,
									$this->mg2->thumbMaxWidth,
									$this->mg2->thumbMaxHeight,
									(bool)($this->mg2->foldersetting & 1<<16)
								 );

		// CREATE THUMB AND VALUE IMAGES
		foreach ($create as $key=>$imgCreateRC) {

			// INIT START VALUES
			$rotateImg = $imgCreateRC[0] & 8 ? true:false;
			$scaleImg  = true;
			$sourceX   = 0;
			$sourceY   = 0;

			// MAX THUMB RESP. MEDIUM IMAGE SIZE
			$maxWidth  = $imgCreateRC[1];
			$maxHeight = $imgCreateRC[2];

			// FORCE THUMB SIZE TO SETTING VALUES
			if ($imgCreateRC[3]) {
				list(
					$sourceX,
					$sourceY,
					$imgWidth,
					$imgHeight
				) = $this->calculateCropValues($maxWidth, $maxHeight, $imgWidth, $imgHeight);
			}
			// ORIGINAL GREATER THAN MAX THUMB RESP. MEDIUM SIZE
			elseif ($imgWidth > $maxWidth || $imgHeight > $maxHeight) {

				// CALCULATE THE SMALLER IMAGE SIDE
				if ($imgWidth < $imgHeight) {
					$maxWidth  = (int)round($imgWidth/$imgHeight * $maxHeight);
				} else {
					$maxHeight = (int)round($imgHeight/$imgWidth * $maxWidth);
				}
			}
			// ROTATION OR CREATE THUMB FROM SMALLER IMAGE
			elseif ($rotateImg || $key === 'thumb') {
				$maxWidth  = $imgWidth;		// use original image width
				$maxHeight = $imgHeight;	// use original image height
				$scaleImg  = false;
			}
			// IMAGE TO SMALL TO SCALE
			else continue;

			// SAVE CALCULATED SIZE FOR RETURN
			$result[$key] = array('width'=>$maxWidth, 'height'=>$maxHeight);

			// DON'T CREATE THUMBS?
			if (!$imgCreateRC[0])										continue;

			// GET IMAGE PATH
			$dstFilePath = $this->mg2->getFilePathByID($itemID, $key);
			if (!$dstFilePath)											continue;	// path error

			// CREATE SOURCE IMAGE
			if (!isset($srcImage)) {

				// MEMORY CONSUMPTION OF UNPACKED IMAGE
				$errorMsg = sprintf(' * Image \'%s\' unpacked too large (%%s) to %s',
									$fileName,
									$rotateImg ? 'rotate':'scale'
								);
				$memConsum = $this->mg2->calculateMemDemand($itemMetrics['itemInfo'], $maxWidth, $maxHeight, $errorMsg);
				if ($memConsum !== true) {
					do {
						do {
							if (!$rotateImg)		break;	// don't rotate image
							if ($scaleImg)			break;	// do scale original

							// IF THE MEM CONSUMPTION MORE THEN 1.4285 TIMES AS THE AVAILABLE MEMORY
							if ((abs($memConsum) / $this->mg2->currentMemory['available']) > 0.4285) {
								break;
							}

							// CALCULATE MEM CONSUMPTION OF THE NEXT CREATE RECORD 
							list($cID, $width, $height) = isset($create['medium'])?
																	$create['medium']
																	:
																	$create['thumb'];
							if ($this->mg2->calculateMemDemand($itemMetrics['itemInfo'], $width, $height) !== true) {
								break;
							}

							// LOAD AND INSTANTIATE ALTERNATIVE ROTATE METHOD
							include_once(INC_FOLDER. 'mg2admin_rotation.php');

							if ($Rotate = new MG2Rotation())	break 2;	// everything OK
						}
						while(0);

						// NOT ENOUGH MEMORY TO PROCESS THE IMAGE
						$errorMsg = $rotateImg ? 'imgrotate_error':'memorytoosmall';
						$result['error'][3] = sprintf($this->mg2->lang[$errorMsg],
														$fileName,
														$this->mg2->convertBytes($this->mg2->currentMemory['limit'], 0)
													 );
						break 2;	// break off image processing
					}
					while(0);
				}

				// CREATE SOURCE IMAGE
				if (!$srcImage = $itemMetrics['createFct']($imgFilePath)) {
					$result['error'][3] = sprintf($this->mg2->lang['imgprocess_error'], $fileName);
					break;
				}
			}

			// CREATE TRUE COLOR IMAGE AND RESAMPLE (SCALE) IT
			if ($scaleImg) {
				$dstImage	= imagecreatetruecolor($maxWidth, $maxHeight);
				$scaleImg	= imagecopyresampled($dstImage, $srcImage, 0, 0, $sourceX, $sourceY, $maxWidth, $maxHeight, $imgWidth, $imgHeight);
				$dstImgName	= $scaleImg ? 'dstImage':'srcImage';
			}
			else {
				$scaleImg	= $key === 'thumb' ? true:false;
				$dstImgName = 'srcImage';
			}

			// ROTATE IMAGE
			do {
				// IF NOT ROTATING
				if (!$rotateImg) break;

				// DEFAULT GDLIB ROTATION
				if (!isset($Rotate)) {
					if ($dstImgName === 'dstImage')	$srcImage = null;
					if (!$srcImage = imagerotate($$dstImgName, $rotateDeg, 0)) {
						$this->mg2->log(' * WARNING: Couldn\'t rotate image by '. $rotateDeg .' deg.');
						$result['error'][$errLevel] = $this->mg2->lang['imagenotrotated']. ' \''. $fileName .'\'';
						$rotateImg = false;
						if ($errLevel === 3) break 2; else break 1;
					}
				}
				// MG2 PARTITION ROTATION
				elseif (!$Rotate->doRotate($srcImage, $rotateDeg)) {
					$this->mg2->log(' * WARNING: Couldn\'t rotate image by '. $rotateDeg .' deg.');
					$result['error'][$errLevel] = $this->mg2->lang['imagenotrotated']. ' \''. $fileName .'\'';
					$rotateImg = false;
					if ($errLevel === 3) break 2; else break 1;
				}

				$dstImgName = 'srcImage';
				unlink($dstFilePath);		// to avoid denied permission
			}
			while(0);

			if ($scaleImg || $rotateImg) {

				// WORKAROUND FOR 'SAFE MODE' BUG
				if (version_compare(PHP_VERSION, '5.0.0', '<')) touch($dstFilePath);

				// CREATE A NEW PICTURE FILE
				if ($itemMetrics['formatFct']($$dstImgName, $dstFilePath, $itemMetrics['imgQuality'])) {
					if ($rotateImg) {
						list($imgHeight, $imgWidth) = array($maxWidth, $maxHeight);
						$result['item']   = array('width'=>$imgWidth, 'height'=>$imgHeight);
						$result['rotate'] = $rotateDeg;
					}
					if     ($scaleImg && ($key === 'thumb'))  $result['scaled'] |= 1;
					elseif ($scaleImg && ($key === 'medium')) $result['scaled'] |= 2;
					elseif ($scaleImg && ($key === 'item'))   $result['scaled'] |= 4;
				}
			}
			if (is_resource($dstImage)) imagedestroy($dstImage);
		}
		if (is_resource($srcImage)) imagedestroy($srcImage);

		// WRITE BACK EXIF TAGS
		if ($exifOK && ($result['rotate'] || ($result['scaled'] & 4))) {
			$this->writeBackExifData($imgFilePath);
		}

		// RETURN IMAGE AND THUMB SIZES
		return $result;
	}

	// GET METRICS OF THE CURRENT ITEM
	function getItemMetrics($imgFilePath) {

		// GET SOURCE ITEM TYPE
		$imageInfo = getimagesize($imgFilePath);			// takes about 0.2 sec. for 50 MB
		$imageType = array(										// video or sound file!
							IMAGETYPE_GIF  =>'gif',
							IMAGETYPE_JPEG =>'jpg',
							IMAGETYPE_PNG  =>'png'
						 );
		if (isset($imageType[$imageInfo[2]])) {
			$fileExt  = $imageType[$imageInfo[2]]; 
			$imageExt = $fileExt;
		}
		else {
			$fileExt  = substr(strrchr($imgFilePath, '.'), 1);
			$imageExt = null;
		}
		$itemCode = $this->mg2->getContentCode($fileExt);

		// BUILD RETURN ARRAY
		$itemMetrics = array (
								'createFct'  => '',
								'formatFct'  => '',
								'imgQuality' => null,
								'itemInfo'   => $imageInfo,
								'itemCode'   => $itemCode,
								'process'    => 127				// permit all possible settings
							);

		// GET APPROPRIATE IMAGE FUNCTIONS AND SETTINGS
		switch ($imageExt) {
			case 'gif' :
				if (imagetypes() & IMG_GIF) {
					$itemMetrics['createFct'] = 'imagecreatefromgif';
					$itemMetrics['formatFct'] = 'imagegif';
				}
				else {
					$this->mg2->log(' * ERROR: GIF files can\'t be processed with the used GD lib!');
					$itemMetrics = false;
				}
				break;
			case 'jpg' :
				$itemMetrics['createFct']  = 'imagecreatefromjpeg';
				$itemMetrics['formatFct']  = 'imagejpeg';
				$itemMetrics['imgQuality'] = abs($this->mg2->thumbquality);
				break;
			case 'png' :
				$itemMetrics['createFct']  = 'imagecreatefrompng';
				$itemMetrics['formatFct']  = 'imagepng';
				$itemMetrics['imgQuality'] = (int)floor(abs($this->mg2->thumbquality/10));
				$itemMetrics['imgQuality'] = min($itemMetrics['imgQuality'], 9);
				break;
			// NO SUPPORTED IMAGE FORMAT
			case null :
				// IMAGE FORMAT 'SVG'
				if (($itemCode & 15) === $this->mg2->contentCodeTable('svg')) {
					$itemMetrics['itemInfo'] = $this->getSizeOfSVG($imgFilePath);
					$itemMetrics['process']  = 0;						// compute thumb size only
					break;
				}
				// META FORMAT 'VIDEO'
				if (($itemCode & 448) === $this->mg2->contentCodeTable('video')) {
					$itemMetrics['itemInfo'] = array(400, 300);	// canvas width and height
					$itemMetrics['process']  = false;				// create no thumb or medium
					break;
				}
				// META FORMAT 'AUDIO'
				if (($itemCode & 448) === $this->mg2->contentCodeTable('audio')) {
					$itemMetrics['itemInfo'] = array(400, 26);	// canvas width and height
					$itemMetrics['process']  = false;				// create no thumb or medium
					break;
				}
			default:	$itemMetrics = false;
		}

		return $itemMetrics;
	}

   /**
	 * Computes the size of a SVG file,
	 * provides with and height in pixels
	 *
	 * @param string $svgFilePath
	 * @return array
	 */
	function getSizeOfSVG($svgFilePath) {

		// LOAD SVG FILE
		$xmlString = file_get_contents($svgFilePath);

		if (extension_loaded('simplexml')) {
			$xmlGrafic  = simplexml_load_string($xmlString);
			$xmlAttribs = $xmlGrafic->attributes();
			$xmlWidth   = (string)$xmlAttribs->width;
			$xmlHeight  = (string)$xmlAttribs->height;
			$xmlUnit    = substr($xmlHeight, -2);
		}
		else {
			$regexp = '/<svg.+width="([0-9.]{1,10})[a-z]{2,2}".+height="([0-9.]{1,10})([a-z]{2,2})".+>/Usi';
			preg_match($regexp, $xmlString, $treffer);
			$xmlWidth  = $treffer[1];
			$xmlHeight = $treffer[2];
			$xmlUnit   = $treffer[3];
		}

		switch ($xmlUnit) {
			case 'mm':
				$xmlWidth  = $xmlWidth * 92/25.4;
				$xmlHeight = $xmlHeight * 92/25.4;
				break;
			case 'cm':
				$xmlWidth  = $xmlWidth * 92/2.54;
				$xmlHeight = $xmlHeight * 92/2.54;
				break;
			case 'in':
				$xmlWidth  = $xmlWidth * 92;
				$xmlHeight = $xmlHeight * 92;
				break;
			case 'pt':
				$xmlWidth  = $xmlWidth * 92/(25.4 * 2.8353);
				$xmlHeight = $xmlHeight * 92/(25.4 * 2.8353);
				break;
		}

		return array (
					(int)$xmlWidth,
					(int)$xmlHeight
				 );
	}

	// CALCULATE CROP VALUES TO CREATE THUMBS
	function calculateCropValues($cropWidth, $cropHeight, $imageWidth, $imageHeight) {

		// CALCULATE CROP AREA
		$cropArea   = 0.96;																	// maximal area for width and height
		$cropBorder = (1 - $cropArea)/2;													// minimal border for crop image

		$ratioW = $cropArea * ($imageWidth/$cropWidth);								// original to crop width
		$ratioH = $cropArea * ($imageHeight/$cropHeight);							// original to crop height

		if ($ratioW < $ratioH) {
			$sourceX		 = floor($imageWidth * $cropBorder);
			$imageWidth	 = round($imageWidth * $cropArea);

			$sourceY		 = floor(($imageHeight - ($cropHeight * $ratioW))/3);	// vertical one third from top
			$imageHeight = round($cropHeight * $ratioW);
		}
		else {
			$sourceY		 = floor($imageHeight * $cropBorder);
			$imageHeight = round($imageHeight * $cropArea);

			$sourceX		 = floor(($imageWidth - ($cropWidth * $ratioH))/2);	// horizontal to center
			$imageWidth  = round($cropWidth * $ratioH);
		}

		return array($sourceX, $sourceY, $imageWidth, $imageHeight);
	}

	// WRITE BACK FXIF SECTION OF JPEG
	function writeBackExifData($imgFilePath) {

		$img  = file_get_contents($imgFilePath);
		$img  = strstr($img, "\xff\xdb");
		$len  = $this->mg2->MG2Exif->dataLength + 2;
		$head = "\xff\xd8\xff\xE0\x00\x10JFIF\x00\x01\x01\x01\x01\x2C\x01\x2C\x00\x00\xff\xe1";
		$head.= sprintf("%c%c", ($len >> 8) & 255, $len & 255);
		if ($fp = fopen($imgFilePath, 'wb')) {
			fwrite($fp, $head);
			fwrite($fp, $this->mg2->MG2Exif->exifBlock);
			fwrite($fp, $img);
			fclose($fp);
		}
	}
}
