<?php
/**
 * Class for image rotation to reduce the
 * memory demand.
 *
 * For that reason the class slices images
 * into several parts, rotates each part and
 * save it as file.
 *
 * To Use this class, e.g.:
 * 
 * $rotateAngle = 90;
 * $imgQuality  = 85;
 * $imgFilePath = 'img/testImg.jpg';
 * $dstFilePath = 'img/testImgRotated.jpg';
 * $srcImage    = imagecreatefromjpeg($imgFilePath);
 * $Rotate      = new MG2Rotation();
 * $Rotate->doRotate($srcImage, $rotateAngle);
 * imagejpeg($srcImage, $dstFilePath, $imgQuality);
 */
class MG2Rotation {

	var $srcImage;

	// CONTROL IMAGE ROTATION
	function doRotate(&$srcImage, $rotAngle, $cycles=6) {

		$this->srcImage = &$srcImage;
		$rotAngle = $this->filterRotAngle($rotAngle);
		$cycles   = $this->filterRotCycles($cycles);
		$imgParts = $this->sliceAndRotate($rotAngle, $cycles);

		return $this->buildRotatedImg($imgParts);
	}

	function filterRotAngle($rotAngle) {

		switch((int)$rotAngle) {
			case   90:
			case -270: $rotAngle =  90; break;	// rotate to left
			case  -90:
			case  270: $rotAngle = -90; break;	// rotate to right
			case  180:
			case -180: $rotAngle = 180; break;
			default: die('This rotation angle isn\'t supported: '. $rotAngle);
		}

		return $rotAngle;
	}

	function filterRotCycles($cycles) {

		return ($cycles < 1)?
				 1
				 :
				 (int)$cycles;
	}

	function sliceAndRotate($rotAngle, $cycles) {

		$imgWidth  = imagesx($this->srcImage);
		$imgHeight = imagesy($this->srcImage);
		$imgParts  = array();
		$src_y     = 0;				// y-Startpunkt "imagecopy"
		$src_x     = 0;				// x-Startpunkt "imagecopy"
		$src_h     = $imgHeight;
		$src_w     = $imgWidth;

		// IF NOT ROTATION BY 180 GRD
		if ((int)$rotAngle % 180) {
			$runDimension = 'src_x';
			$runThickness = 'src_w';
			list($imgHeight, $imgWidth) = array($imgWidth, $imgHeight);
		}
		// IF ROTATION BY 180 GRD
		else {
			$runDimension = 'src_y';
			$runThickness = 'src_h';
		}
		$partThickness = (int)floor($imgHeight/$cycles);

		for ($i=0; $i<$cycles; $i++) {

			$$runDimension = $partThickness * $i;
			$$runThickness = ($i + 1 === $cycles)?
								  $imgHeight - $$runDimension
								  :
								  $partThickness;

			$partial = imagecreatetruecolor($src_w, $src_h);
			imagecopy($partial, $this->srcImage, 0, 0, $src_x, $src_y, $src_w, $src_h);
			$partial = imagerotate($partial, $rotAngle, 0);

			$imgParts[$i] = tempnam('quark', 'mg2');
			imagepng($partial, $imgParts[$i]);
		}

		// IF IMAGE ROTATION TO LEFT
		if ($rotAngle > 0) {
			$imgParts = array_reverse($imgParts);
		}

		// IF NOT ROTATION BY 180 GRD
		if ((int)$rotAngle % 180) {
			$this->srcImage = null;
			$this->srcImage = imagecreatetruecolor($imgWidth, $imgHeight);
		}

		return $imgParts;
	}

	// BUILD NEW IMAGE
	function buildRotatedImg($imgParts) {

		$dest_y = 0;
		$done   = 0;
		$cycles = count($imgParts);
		for ($i=0; $i<$cycles; $i++) {

			$partial = null;
			if (!$partial = imagecreatefrompng($imgParts[$i])) continue;

			imagecopy(	$this->srcImage,
							$partial,
							0,
							$dest_y,
							0,
							0,
							imagesx($partial),
							imagesy($partial)
						);
			$dest_y += imagesy($partial);
			$done++;
		}

		return $cycles === $done ? true:false;
	}
}
