<?php
//
// STATS CLASS (ADMIN ONLY)
// kh_mod 0.4.0 rc1, add
class MG2Stats extends MG2Counter {

	var $folderNumClicks;
	var $folderTimestamps;

	// INITIALIZE STATS
	function MG2Stats(&$parent) {
		$this->MG2Counter($parent, false);
		$this->delResidueCounterData();
		$this->delResidueChronicleData();
		$this->optimizeChronicleData();
		$this->buildFolderCounterData();
		settype($this->mg2->statisticsets, 'integer');
	}

	// BUILD FOLDER - COUNTER DATA RELATIONS
	function buildFolderCounterData() {

		$this->folderNumClicks  = array();
		$this->folderTimestamps = array();
		foreach ($this->counterData as $counterRC) {
			$itemID	 = (int)$counterRC[0];
			$folderID = (int)$this->mg2->all_items[$itemID][1];
			if (($numberClicks = (int)$counterRC[1]) < 1) continue;
			if (($latestClick  = (int)$counterRC[3]) < 1) continue;

			if (isset($this->folderNumClicks[$folderID])) {
				$this->folderNumClicks[$folderID]+= $numberClicks;
			}
			else {
				$this->folderNumClicks[$folderID] = $numberClicks;
			}
			$this->folderTimestamps[$folderID][$itemID] = $latestClick;
		}
	}

	// IF TIME DISPLAY LOCKED THAN SET DEFAULT VALUE
	function setDefaultTimeFormat() {
		if (!$this->mg2->timeformat) {
			$this->mg2->timeformat = $this->mg2->getTimeFormat('default');
		}
	}

	// GET USED MEMORY OF DIRECTORY, INC. SUB DIRS
	function getDirSize($dirName) {

		$dirSize = 0;
		if (is_dir($dirName)) {
			if ($dh = opendir($dirName)) {
				while (($file = readdir($dh)) !== false) {
					if($file === '.' || $file === '..') continue;

					if(is_file($dirName .'/'. $file)) {
						$dirSize += filesize($dirName .'/'. $file);
					}
					// CHECK ALSO ALL SUB DIRECTORIES
					if(is_dir($dirName .'/'. $file)) {
						$dirSize +=  $this->getDirSize($dirName .'/'. $file);
					}
				}
				closedir($dh);
			}
		}
		return $dirSize;
	}

	// GET NUMBER OF RECORDS TO DISPLAY (FORM)
	function getDisplNumber($displName) {

		// CHECK '$displName' AND SET THE BIT AREA
		switch ($displName) {
			case 'numLatestClicks':
				$startBit =  8; $bitRange = 3 << $startBit; break;
			case 'numMostItems':
				$startBit = 10; $bitRange = 3 << $startBit; break;
			case 'numMostFolders':
				$startBit = 12; $bitRange = 3 << $startBit; break;
			case 'numLatestComm':
				$startBit = 14; $bitRange = 3 << $startBit; break;
			case 'numLatestItems':
				$startBit = 16; $bitRange = 3 << $startBit; break;
			case 'numLatestFolders':
				$startBit = 18; $bitRange = 3 << $startBit; break;
			default;
				return false;
		}

		// VALUES FOR 'displNumber'
		$numCodeToValue = array (5, 10, 20, 50);

		// SETTINGS CHANGED?
		$setNumCode = ($this->mg2->statisticsets & $bitRange) >> $startBit;
		do {
			if (!isset($_POST[$displName]))	break;

			$newNumCode = array_search((int)$_POST[$displName], $numCodeToValue, true);
			if ($newNumCode === false)			break;
			if ($newNumCode === $setNumCode)	break;

			$setNumCode						= $newNumCode;
			$this->mg2->statisticsets &= ~$bitRange;
			$this->mg2->statisticsets |= $newNumCode << $startBit;
		}
		while(0);

		return isset($numCodeToValue[$setNumCode]) ?
				 $numCodeToValue[$setNumCode]
				 :
				 false;	// use default setting
	}

	// GET PAST TIME STAMP FOR ITEMS OR FOLDERS (FORM)
	function getTimePeriod($periodName) {

		// CHECK '$periodName' AND SET THE BIT AREA
		switch ($periodName) {
			case 'periodClickStat':
				$startBit = 0; $bitRange = 3 << $startBit; break;
			case 'periodItems':
				$startBit = 2; $bitRange = 7 << $startBit; break;
			case 'periodFolders':
				$startBit = 5; $bitRange = 7 << $startBit; break;
			default;
				return false;
		}

		// VALUES FOR 'timePeriod'
		$periodCodeToValue = array (
										0	=>	'tenDays',		// the last 10 days
										1	=> 'lastDays',		// the last 28 days
										2	=> 'thisMonth',	// the current month
										3	=> 'thisYear',		// the current year
										4	=> 'total'			// entire counter period
									);

		// SETTINGS CHANGED?
		$setPeriodCode = ($this->mg2->statisticsets & $bitRange) >> $startBit;
		do {
			if (!isset($_POST[$periodName]))			break;

			$newPeriodCode = array_search($_POST[$periodName], $periodCodeToValue, true);
			if ($newPeriodCode === false)				break;
			if ($newPeriodCode === $setPeriodCode)	break;

			$setPeriodCode					= $newPeriodCode;
			$this->mg2->statisticsets &= ~$bitRange;
			$this->mg2->statisticsets |= $newPeriodCode << $startBit;
		}
		while(0);

		return ($setPeriodCode < 4)?
				 $periodCodeToValue[$setPeriodCode]
				 :
				 false;	// use default setting
	}

	// GET DATE OF THE LATEST CLICK - FOLDER
	function getLatestFolderClick($folderID) {

		// INIT RETURN VALUE
		$date = '&ndash;';

		if (!empty($this->folderTimestamps[$folderID])			&&
			($timestamp = max($this->folderTimestamps[$folderID])))
		{
			$date = $this->mg2->time2date($timestamp);
		}

		return $date;
	}

	// GET DATE OF THE LATEST CLICK - ITEM
	function getLatestItemClick($itemID, $showTime=true) {

		// INIT RETURN VALUE
		$date = '&ndash;';

		if (isset($this->counterData[$itemID])		&&
			 $this->counterData[$itemID][1] > 0		&&		// number of clicks
			 $this->counterData[$itemID][3] > 0)			// latest time stamp
		{
			$date = $this->mg2->time2date($this->counterData[$itemID][3], $showTime);
		}

		return $date;
	}

	// GET TIME OF LATEST DAILY CLICK OF AN ITEM
	function getLatestDailyClick($itemID, $timestamp) {

		if (!$this->loadChronicleData()) return false;

		$date  = date('Ymd', $timestamp);
		$last  = isset($this->chronicleData['daily'][$date]['lastClick'][$itemID]) ?
					$this->chronicleData['daily'][$date]['lastClick'][$itemID]
					:
					-1;

		return $this->mg2->time2date($last, true, false);
	}

	// GET NUMBER OF DAILY CLICKS FOR $period
	function getNumDailyClicks($period='tenDays') {

		if (!$this->loadChronicleData()) return false;

		// SET INIT VALUES
		$return  = new stdClass();
		$clicks  = array();
		$latest  = array();
		$date    = date('Ymd');
		$m       = substr($date, 4, 2);
		$d       = substr($date, 6, 2);
		$Y       = substr($date, 0, 4);
		$daysMap = array(	'thisMonth' => (int)$d,
								'lastDays'  => 28,
								'tenDays'   => 10
					  );
		if (!isset($daysMap[$period])) $period = 'tenDays';

		for ($i=0; $i<$daysMap[$period]; $i++) {

			$mktime = mktime(0, 0, 0, (int)$m, $d-$i, (int)$Y);
			$date   = date('Ymd', $mktime);

			$clicks[$mktime] = 0;
			$latest[$mktime] = '&ndash;';

			// IS SET "itemClicks"
			if (isset($this->chronicleData['daily'][$date]['itemClicks'])) {
				$clicks[$mktime] = array_sum($this->chronicleData['daily'][$date]['itemClicks']);

				// GET LATEST TIMESTAMP
				$latestTimestamp = !empty($this->chronicleData['daily'][$date]['lastClick']) ?
										 max($this->chronicleData['daily'][$date]['lastClick'])
										 :
										 false;

				// IS SUM OF "itemClicks" AND "lastClick" GREATER THAN 0
				if ($clicks[$mktime]	&& $latestTimestamp)	{
					$latest[$mktime] = $this->mg2->time2date($latestTimestamp, true, false);
				}
			}
		}

		$return->itemClicks  = $clicks;
		$return->latestClick = $latest;
		$return->maxClicks   = max($clicks);

		return $return;
	}

	// GET CLICK DETAILS OF ONE DAY
	function getDailyDetails($timestamp) {

		if (!$this->loadChronicleData()) return false;

		$date = date('Ymd', $timestamp);
		return isset($this->chronicleData['daily'][$date]['itemClicks']) ?
				 $this->chronicleData['daily'][$date]['itemClicks']
				 :
				 array();
	}

	// GET ITEMS THE LATEST OR MOST CLICKS	
	function getNumRatedClicks($ratedMode, $numItems='all') {

		// SET RETURN VALUE
		$ratedClicks = array();

		// CHOOSE RATED FEATURE
		if 	 ($ratedMode === 'latest')	$cols = 3; // sort by date
		elseif ($ratedMode === 'most')	$cols = 1; // sort by clicks
		else return $ratedClicks;

		// SORT COUNTER DATA
		$this->mg2->sortNumeric($this->counterData, $cols, 1);
		$itemIDs = array_keys($this->counterData);

		// CUT OFF THE WANTED NUMBER OF ITEMS
		if ($numItems !== 'all') {
			$itemIDs = array_slice($itemIDs, 0, (int)$numItems);
		}

		// GET LATEST COUNTER RECORDS
		foreach ($itemIDs as $itemID) {
			if (($clicks = (int)$this->counterData[$itemID][1]) < 1) break;

			$ratedClicks[$itemID] = $clicks;
		}

		return $ratedClicks;
	}

	// GET THE LATEST FOLDERS
	function getNumFolderClicks($numLatestFolders=0) {

		// SET INIT VALUES
		$folderClicks  = array();
		$allFolderIDs  = array_keys($this->mg2->all_folders);
		$this->mg2->sortIndexDB($allFolderIDs, 'all_folders', 20);
		$latestFolders = array_slice($allFolderIDs, 0, $numLatestFolders);

		foreach($latestFolders as $folderID) {

			$folderClicks[$folderID] = isset($this->folderNumClicks[$folderID]) ?
												array(
													$this->folderNumClicks[$folderID],
													max($this->folderTimestamps[$folderID])
												)
												:
												array(0, -1);
		}

		return $folderClicks;
	}

	// GET THE MOST CLICKED FOLDERS
	function getNumMostFolderClicks($numFolders='all') {

		// SORT FOLDER CLICKS
		arsort($this->folderNumClicks, SORT_NUMERIC);

		// CUT OFF THE WANTED NUMBER OF ITEMS
		$folderIDs = ($numFolders === 'all')?
						 array_keys($this->folderNumClicks)
						 :
						 array_slice(array_keys($this->folderNumClicks), 0, (int)$numFolders);

		// BUILD RETURN ARRAY
		$return = array();
		foreach($folderIDs as $folderID) {
			if (($clicks = $this->folderNumClicks[$folderID]) < 1) break;

			$return[$folderID] = $clicks;
		}

		return $return;
	}

	// GET ITEMS WITH RECENT CLICKS	
	function getNumRecentClicks($timePeriod, $numItems=5, $isFolders=false) {

		$getNumRecentClicks = ($this->type === 'sql')?
									 'getNumRecentClicksSQL'
									 :
									 'getNumRecentClicksFlatfile';
		return $this->$getNumRecentClicks($timePeriod, $numItems, $isFolders);
	}

	// GET ITEMS WITH RECENT CLICKS - SQL
	function getNumRecentClicksSQL($timePeriod, $numItems, $isFolders) {

		// SET INIT VALUES
		$recentClicks = array();
		$recentDate   = $this->getRecentDate($timePeriod);
		$dateCode	  = ($timePeriod !== 'thisYear')? 1:3;
		$groupBy		  = $isFolders ? 'folderID':'itemID';

		$query = 'SELECT t2.'.$groupBy.', sum(t1.itemClicks) `clicks`
					 FROM `mg2db_chronicle` t1, `mg2db_items` t2
					 WHERE t1.date     >= "'.$recentDate.'"
					 AND   t1.dateCode  &  '.$dateCode.'
					 AND   t1.itemID    = t2.itemID
					 GROUP BY t2.'.$groupBy.'
					 ORDER BY `clicks` DESC, `timestamp` DESC
					 LIMIT '. (int)$numItems;

		if (!$result = mg2mysql_query($query)) return false;

		while ($record = mg2mysql_fetch_row($result)) {
			$recentClicks[(int)$record[0]] = (int)$record[1];
		}

		return $recentClicks;
	}

	// GET ITEMS WITH RECENT CLICKS - FLATFILE
	function getNumRecentClicksFlatfile($timePeriod, $numItems, $isFolders=false) {

		// SET INIT VALUES
		$return		  = array();
		$recentClicks = array();
		$dates        = array();
		$recentDate   = $this->getRecentDate($timePeriod);

		if (!$this->loadChronicleData()) return $return;

		// GET REQUESTED RECENT DATES
		foreach (array_keys($this->chronicleData['daily']) as $dailyDate) {
			if ($recentDate > $dailyDate) continue;

			$dates[$dailyDate]  = 'daily';
		}
		if ($timePeriod === 'thisYear' && isset($this->chronicleData['year'][$recentDate])) {
			$dates[$recentDate] = 'year';
		}

		// COUNT CLICKS OF RECENT DATES
		foreach($dates as $date=>$dateType) {
			foreach($this->chronicleData[$dateType][$date]['itemClicks'] as $itemID=>$clicks) {
				$groupBy = $isFolders ?
							  (int)$this->mg2->all_items[$itemID][1]
							  :
							  $itemID;
				if (isset($recentClicks[$groupBy])) {
					$recentClicks[$groupBy]+= $clicks;
				}
				else {
					$recentClicks[$groupBy] = $clicks;
				}
			}
		}
		arsort($recentClicks, SORT_NUMERIC);

		// CUT OFF THE WANTED NUMBER OF ITEMS
		$groupByIDs = array_slice(array_keys($recentClicks), 0, (int)$numItems);
		foreach($groupByIDs as $groupByID) {
			$return[$groupByID] = $recentClicks[$groupByID];
		}

		return $return;
	}

	// GET FORMATTED DAILY DATE
	function getDailyDate($dailyTime) {

		// INIT START VALUES
		$tDiff = time() - $dailyTime;
		$label = '';
		$date  = $this->mg2->time2date($dailyTime, false);

		if ($tDiff < 0) {
			// date in future
		}
		elseif ($tDiff  < 86400) {
			$label = $this->mg2->lang['today'];
		}
		elseif ($tDiff < 172800) {
			$label = $this->mg2->lang['yesterday'];
		}
		elseif ($tDiff < 259200) {
			$label = $this->mg2->lang['twodaysago'];
		}

		return sprintf($label ? '%s (%s)':'%s', $date, $label);
	}
}
?>