<?php
//
// ADMIN VIEWTABLE HELPER CLASS, IMPLEMENTATION SHOULD BE A STATIC CLASS
// kh_mod 0.4.0 rc1, add
class MG2ViewtableHelper {

	var $mg2;					// parent object reference
	var $folderID;				// folder id

	// INITIALIZE VIEWTABLE
	function MG2ViewtableHelper(&$parent, $folderID=1) {
		$this->mg2		 = &$parent;
		$this->folderID = $folderID;
	}

	// BUILD FOLDER STRUCTURE AND ALLOCATE ITEMS
	function buildDataStructure() {
		if (!isset($this->mg2->folderStructure) || !isset($this->mg2->itemAllocation)) {
			$this->mg2->folderStructure = $this->mg2->getValidSubfolders($this->folderID);
			$this->mg2->itemAllocation  = $this->mg2->getItemAllocation($this->mg2->folderStructure);
		}
	}

	// SUB FOLDER IDs OF THE CURRENT FOLDER
	function getSubFolderIDs() {
		return array_diff(
					 array_keys($this->mg2->folderStructure),
					 array($this->folderID)
				 );
	}

	// ITEM IDs OF THE CURRENT FOLDER
	function getItemIDs() {
		return (array_key_exists($this->folderID, $this->mg2->itemAllocation))?
				 array_keys($this->mg2->itemAllocation[$this->folderID])
				 :
				 array();
	}

	// GET SORT NAVIGATION STRING
	function getNavigationString($totalItems, $numPages, $currentPage) {

		// FOLDER PASSWORD SET?
		$folderPwd = (!empty($this->mg2->all_folders[$this->folderID][8]))?
						 sprintf('<img src="%1$slock.gif" width="15" height="15" alt="%2$s" title="%2$s" style="margin-left:.5em;vertical-align:text-bottom" />',
							 ADMIN_IMAGES,
							 $this->mg2->lang['thissection']
						 )
						 :
						 '';

		// BUILD BREAD CRUMB NAVIGATION
		return sprintf('%s : <a href="?fID=%d&amp;page=%s&amp;display=sortview" style="font-style:italic;">%s</a> : %s&nbsp;%s%s%s',
					 $this->mg2->adminNavigation($this->folderID),
					 $this->folderID,
					 $currentPage,
					 $this->mg2->lang['sorting'],
					 $totalItems,
					 $this->mg2->lang['images'],
					 $this->mg2->pageNavigation($this->folderID, $numPages, $currentPage),
					 $folderPwd
				 );
	}

	function displayFolderRows($folderIDs, $currentPage=1, $displayType='default') {

		// DISPLAY FOLDER ROWS
		foreach($folderIDs as $subfID) {
			$numSubItems = isset($this->mg2->itemAllocation[$subfID]) ?
								count($this->mg2->itemAllocation[$subfID])
								:
								0;
			$folderRC	 = $this->mg2->all_folders[$subfID];
			$folderName  = $this->mg2->htmlchars2entities($folderRC[2]);
			$folderFlag  = (trim($folderRC[8]) != '')?	1:0;	// password
			$folderFlag |= ((int)$folderRC[6] > 0)?	 	2:0;	// thumb id

			// GET SMALL FOLDER ICON
			if ($folderFlag === 3) {		// is set folder password and thumb?
				$small_icon = ADMIN_IMAGES .'folder_small_thumb_locked.gif';
			}
			elseif ($folderFlag === 1) {	// is set folder password only?
				$small_icon = ADMIN_IMAGES .'folder_small_locked.gif';
			}
			elseif ($folderFlag === 2) {	// is set folder thumb only?
				$small_icon = ADMIN_IMAGES .'folder_small_thumb.gif';
			}
			else {								// use standard icon
				$small_icon = ADMIN_IMAGES .'folder_small.gif';
			}

			// GET PUBLISH DATE AND TIME OF FOLDER
			$publishDate = $this->mg2->time2date($folderRC[4], false);
			$publishTime = (empty($this->mg2->timeformat))?
								''
								:
								$this->mg2->time2date($folderRC[4]);

			// GET FOLDER STATUS FOR BACKGROUND COLOR
			if ((int)$folderRC[5] < 0) {		// sub folder locked?
				$bgcolor = sprintf(' bgcolor="#FFCFCF" title="%s (%s %d)"',
									$this->mg2->lang['nodisplay'],
									$this->mg2->lang['position'],
									$folderRC[5]
							  );
			}
			elseif ((int)$folderRC[4] > time()) {	// date in future?
				$bgcolor = sprintf(' bgcolor="#FFFF99" title="%s %s"',
									$this->mg2->lang['notpublished'],
									$publishDate
							  );
			}
			else {
				$bgcolor = '';
			}

			// FOLDER TOOLTIP
			$tooltip = '';
			if (($this->mg2->extendedset & 8)   &&		// displaying switched on?
				 ($thumbID = (int)$folderRC[6]) > 0)	// is set folder thumbnail?
			{
				$thumbFilePath = $this->mg2->getFilePathByID($thumbID, 'thumb');
				if (is_readable($thumbFilePath))
				{
					$tooltip = sprintf('&lt;img src=\\\'%s\\\' width=\\\'%d\\\' height=\\\'%d\\\' alt=\\\'\\\' /&gt;',
										$thumbFilePath,
										$this->mg2->all_items[$thumbID][10],
										$this->mg2->all_items[$thumbID][11]
								  );
				}
			}

			// INCLUDE FOLDER RECORD TEMPLATE
			include(ADMIN_FOLDER .'admin_folders.php');
		}
	}

	// CREATE AND DISPLAY ALL ITEM ROWS
	function displayItemRows($itemIDs, $currentPage=1, $type='default') {

		// LOAD AND INIT EXIF CLASS
		include_once(INC_FOLDER .'mg2_exif.php');
		$Exif = new MG2Exif();

		// THUMB SUFFIX IF DIALOG EDIT ID
		$isuffix	 = (isset($_GET['display']) && ($_GET['display'] === 'editItem'))?
						sprintf('?%d', mt_rand(0,10000))
						:
						'';

		// CONTROL SETTINGS
		if ($type === 'bookmarked') {
			$display      = 'bookmarked';
			$dropIcon     = 'remove';
			$dropControl  = 'action=removeItemLink';
			$dropConfirm  = ' onclick="return confirmRemove(\''. $numRow .'\')"';
		}
		elseif ($type === 'lastimported') {
			$display      = 'lastimported';
			$dropIcon     = 'delete';
			$dropControl  = 'dialog=deleteItem';
			$dropConfirm  = '';
		}
		else {
			$display      = 'tableview';
			$dropIcon     = 'delete';
			$dropControl  = 'dialog=deleteItem';
			$dropConfirm  = '';
		}
		$editIcon      = 'edit';
		$editIconTitle = $this->mg2->htmlchars2entities($this->mg2->lang['edit']);
		$dropIconTitle = $this->mg2->htmlchars2entities($this->mg2->lang[$dropIcon]);

		// DISPLAY ITEM ROWS
		foreach($itemIDs as $numRow=>$itemID) {

			// GET ITEM VALUES
			$itemRC		  = $this->mg2->all_items[$itemID];
			$itemName	  = $this->mg2->htmlchars2entities($itemRC[6]); 
			$itemSubdir	  = $this->mg2->htmlchars2entities($itemRC[7]);
			$itemFilePath = $this->mg2->getFilePathByName($itemRC[6], $itemRC[7]);
			$itemFileText = $this->mg2->getFilePathByName($itemName, $itemSubdir);
			$itemFileSize = $this->mg2->convertBytes((int)$itemRC[12], 1);

			// GET PUBLISH DATE AND TIME OF FILE
			$publishDate = $this->mg2->time2date($itemRC[4], false);
			$publishTime = (empty($this->mg2->timeformat))?
								''
								:
								$this->mg2->time2date($itemRC[4]);

			// GET ITEM STATUS FOR BACKGROUND COLOR
			if ($itemRC[5] < 0)				// item locked?
				$bgcolor = sprintf(' bgcolor="#FFCFCF" title="%s (%s %d)"',
									$this->mg2->lang['nodisplay'],
									$this->mg2->lang['position'],
									$itemRC[5]
							  );
			elseif ($itemRC[4] > time())	// date in future?
				$bgcolor = sprintf(' bgcolor="#FFFF99" title="%s %s"',
									$this->mg2->lang['notpublished'],
									$publishDate
							  );
			else
				$bgcolor = '';

			// GET THUMB FILE
			list(
				$thumbID,
				$thumbFile,
				$thumbWidth,
				$thumbHeight
			) = $this->mg2->getFileIcon($itemID);

			// ADD THUMB FILE SUFFIX
			$thumbFile.= $isuffix; 

			// CALCULATE MINITHUMB SIZE, MINIMAL 5 PIXEL
			$miniThumbWidth  = max((int)round($thumbWidth/5,  0), 5);
			$miniThumbHeigth = max((int)round($thumbHeight/5, 0), 5);

			// GET THUMB TOOLTIP
			$tooltip = '';
			if (($this->mg2->extendedset & 8) && $thumbID !== -1) {
				$thumbImage = '&lt;img src=\\\'%s\\\' width=\\\'%d\\\' height=\\\'%d\\\' alt=\\\'\\\' /&gt;';
				$thumbTitle = '';
				if (($itemID === $thumbID) && $Exif) { // if content type 'image'
					$exifData	= $Exif->getExifData($itemFilePath);
					$exifDate   = (isset($exifData['DTOpticalCapture']))?
									  $this->mg2->exif2date($exifData['DTOpticalCapture'])
									  :
									  '&ndash;';
					$thumbTitle = '&lt;div style=\\\'white-space:nowrap;\\\'&gt;ID: #%d,&nbsp;%s: ';
					$thumbTitle.= (strlen($exifDate) > 7)?
									  '&lt;/div&gt;&lt;div style=\\\'white-space:nowrap;\\\'&gt;'
									  :
									  '&nbsp;';
					$thumbTitle.= '%s&lt;/div&gt;';
				}
				$tooltip = sprintf($thumbImage . $thumbTitle,
									$thumbFile,
									$thumbWidth,
									$thumbHeight,
									$itemID,
									$this->mg2->lang['exif_date'],
									$exifDate
							  );
			}

			// GET NUMBER OF COMMENTS
			$Comments = $this->mg2->getInstance('MG2Comments');
			$commInfo = ($numComments = $Comments->getNumComments($itemID))?
							vsprintf('%d/%d', $numComments)
							:
							'-/-';

			// INCLUDE FILE RECORD TEMPLATE
			include(ADMIN_FOLDER .'admin_files.php');
		}
	}
}
?>