<?php
//
// BUILD DIRECTORY AND FOLDER TREES
// kh_mod 0.4.1, add
class MG2Treeview {

	var $mg2;						// parent object reference
	var $sortedMG2FolderList;	// sorted list of folders

	// INITIALIZE ADMIN VIEW HELPER
	function MG2Treeview(&$parent) {
		$this->mg2 = &$parent;
	}

	// BUILD EXPLORER TREE OF THE ITEMS DIRECTORY
	function buildExplorer($directories) {

		if (!$directories) return array();

		$blr  = '&nbsp;&#9492;';	// BOX DRAWINGS LIGHT UP AND RIGHT
		$blv  = '&nbsp;&#9474;';	// BOX DRAWINGS LIGHT VERTICAL
		$blvr = '&nbsp;&#9500;';	// BOX DRAWINGS LIGHT VERTICAL AND RIGHT
		$tab  = '&nbsp;&nbsp;';

		$reverse   = array_reverse($directories, true);
		$explorer  = array();
		$drawPos   = array();
		$lastLevel = 999999;

		foreach($reverse as $key=>$dir) {
			$dirLevel = $dir['level'];

			// IF TOP LEVEL
			if ($dirLevel < 1) {
				$explorer[$key] = array($dir['name']);
				continue;
			}

			// SUB DIRECTORIES
			$explorer[$key]	= array_fill(0, $dirLevel, $tab);
			$explorer[$key][]	= ($lastLevel === $dirLevel)?
									  $blvr
									  :
									  $blr;
			$explorer[$key][]	= $dir['name'];

			// SET BOX DRAWINGS LIGHT VERTICAL (RIGHT)
			if ($dirLevel > $lastLevel) {													// shift right
				$drawPos[$lastLevel] = $blv;
			}
			elseif ($dirLevel < $lastLevel && isset($drawPos[$dirLevel])) {	// shift left
				$explorer[$key][$dirLevel] = $blvr;
				array_pop($drawPos);
			}
			foreach ($drawPos as $pos=>$box) $explorer[$key][$pos] = $box;

			$lastLevel = $dirLevel;
		}
		return $explorer;
	}

	// GET ITEMS DIRECTORY BRANCH
	function getItemsDirs() {

		// BUILD ITEM DIR RECORD
		$itemDirs[] = array(
							'path'	=> '',
							'level'	=> 0,
							'name'	=> basename($this->mg2->imagefolder),
							'wrable'	=> is_writeable($this->mg2->imagefolder)
						  );

		// IF THERE ARE SUBDIRS
		if ($subdirs = $this->getItemsSubdirs()) {
			uksort($subdirs, array($this, 'natCaseCmp'));
			$itemDirs = array_merge($itemDirs, $subdirs);
		}

		return $itemDirs;
	}

	// NATURAL COMPARE OF SUB DIRECTORIES
	function natCaseCmp($a, $b) {
		return strnatcasecmp($a, $b);
	}

	// GET ITEMS SUBDIRS (RECURSIVE)
	function getItemsSubdirs($path='', $level=0) {
		$level++;
		$subdirs  = array();
		$thispath = $this->mg2->imagefolder . $path;
		$fp = opendir($thispath);
		while ($node = readdir($fp)) {
			$subpath = $path .'/'. $node;
			$newpath = $thispath .'/'. $node;

			// EXECLUSION
			if ($node{0} === '.')	continue;
			if (!is_dir($newpath))	continue;

			$subdirs[$subpath] = array(
				'path'	=> $subpath,
				'level'	=> $level,
				'name'	=> $node,
				'wrable'	=> is_writeable($newpath)
			);
			$subdirs = array_merge($subdirs, $this->getItemsSubdirs($subpath, $level));
		}
		closedir($fp);
		return $subdirs;
	}

	// BUILD OPTION TAGS FOR SELECT BOX
	function getFolderOptionTags($currentfID, $dialog) {

		// INIT START VALUES
		$this->buildFolderlist($currentfID);
		$optionTags  = array();
		$noSubdirs   = false;		// don't show subdirs beneath $currentfID	
		$selectedfID = false;		// preselect option tag with this ID
		$exceptfID   = false;		// don't show the folder with this ID

		if ($dialog === 'upload' || $dialog === 'import') {
			$selectedfID = $currentfID;
		}
		elseif ($dialog === 'editItem') {
			$exceptfID = 1;
		}
		elseif ($dialog === 'editFolder') {
			$noSubdirs = true;
			$exceptfID = (int)$this->mg2->all_folders[$currentfID][1];
		}
		elseif ($dialog === 'controls') {
			$exceptfID = $currentfID;
		}

		foreach ($this->sortedMG2FolderList as $pathID=>$folderInfo) {

			// EXCLUSION CRITERIONS
			if ($noSubdirs && ($folderInfo['state'] & 16))	continue;	// any folder beneath $currentfID	
			if ($exceptfID && ($exceptfID === $pathID))		continue;

			$optionTags[] = $this->builtOptionTag($pathID, $folderInfo, $selectedfID);
		}

		return implode("\n", $optionTags);
	}

	// BUILD OPTION TAG
	function builtOptionTag($pathID, $folderInfo, $selectedfID) {

		// INIT START VALUES
		$style = array();
		$title = array();

		// GET FOLDER STATUS
		if ($folderInfo['state'] & 2) {					// folder locked?
			$style[] = 'background-color: #FF9999;';
			$title[] = $this->mg2->lang['nodisplay'];
		}
		elseif ($folderInfo['state'] & 1) {				// folder published not yet?
			$style[] = 'background-color: #FFFF99;';
			$title[] = sprintf(
								'%s %s',
								$this->mg2->lang['notpublished'],
								$this->mg2->time2date($folderInfo['time'])
						  );
		}
		if (($folderInfo['state'] & 12) === 12) {		// folder icon and/or password set?
			$style[] = 'background-image: url('. ADMIN_IMAGES .'thumb_lock.gif);';
			$style[] = 'background-repeat: no-repeat;';
			$style[] = 'background-position: right;';
			$title[] = $this->mg2->lang['presentation'];
			$title[] = $this->mg2->lang['thissection'];
		}
		elseif ($folderInfo['state'] & 4) {				// folder icon set?
			$style[] = 'background-image: url('. ADMIN_IMAGES .'thumb.gif);';
			$style[] = 'background-repeat: no-repeat;';
			$style[] = 'background-position: right;';
			$title[] = $this->mg2->lang['presentation'];
		}
		elseif ($folderInfo['state'] & 8) {				// folder password set?
			$style[] = 'background-image: url('. ADMIN_IMAGES .'lock.gif);';
			$style[] = 'background-repeat: no-repeat;';
			$style[] = 'background-position: right;';
			$title[] = $this->mg2->lang['thissection'];
		}

		// BUILD OPTION TAG TEMPLATE
		return sprintf(
					 '%s<option style="padding:1px 12px 1px 3px;%s" title="%s" value="%d"%s>%s</option>',
					 "\t\t\t",
					 implode('', $style),
					 implode('; ', $title),
					 $pathID,
					 $pathID === $selectedfID ? ' selected="selected"':'',
					 $folderInfo['path']
				 );
	}

	// BUILD FOLDER TREE
	function buildFolderlist($currentID=-1) {

		// FOLDERLIST DOSE ALREADY EXIST
		if (!empty($this->sortedMG2FolderList)) return;

		settype($currentID, 'integer');
		$this->sortedMG2FolderList = array();
		foreach ($this->mg2->all_folders as $key=>$record) {
			$folderID     = $key;
			$folderPath   = array();
			$folderState  = ((int)$record[4] > time())?	1:0;	// date in future?
			$folderState |= ((int)$record[5] < 0)?			2:0;	// folder locked?
			$folderState |= ((int)$record[6] > 0)?			4:0;	// folder icon set?
			$folderState |= (!empty($record[8]))?			8:0;	// password set?

			// GET PATH TO ROOT FOR EACH FOLDER
			while ($folderID > 1) {
				if ($folderID === $currentID) $folderState |= 16;							// current or sub folder
				$folderPath[$folderID] = $this->mg2->getFolderName($folderID, true);	// folder name
				$folderID				  = (int)$this->mg2->all_folders[$folderID][1];	// parent folder id
				if (isset($folderPath[$folderID])) break;										// circle data structure
			}
			$fullPath = implode(' : ', array_reverse($folderPath));

			// BUILD FOLDER ENTRY
			$this->sortedMG2FolderList[$key] = array(
															 'path'  => $fullPath,
															 'time'  => $record[4],
															 'state' => $folderState
														  );
		}
		// SORT ALL FOLDER ENTRIES
		asort($this->sortedMG2FolderList);

		//SET ROOT NAME
		if ($this->sortedMG2FolderList[1]) {
			$this->sortedMG2FolderList[1]['path'] = $this->mg2->lang['root'];
		}
	}
}
?>